//////////////////////////////////////////////////////////////////////////
/// \file	CollisionSphere.cpp
/// \author	Robert Bateman
/// \brief	Maya contains a number of 'render' primitives, ie, kRenderSphere,
/// 		kRenderBox, kRenderCone etc. These are really useful for the 
/// 		placement of collision objects around the maya scene. They can
/// 		also be used for a host of other purposes - level triggers and 
/// 		switches, fog volumes etc.
///
/// \note	
/// 		This sourcefile is provided for free use for non-commercial purposes only. The
/// 		Software is provided 'as is' and no responsibility can be taken for any damages
/// 		that may be caused while using it. If you make use of this software within 
/// 		your code then it would be nice to have a small credit somewhere in your
/// 		finished product for my hard work :(
/// 		An e-mail saying you found it useful and possibly even a link to your product
/// 		would be really nice too ;)
///
///			Slip me a couple of quid and i might be kind enough to offer assistance in
/// 		sorting out your exporters ;) 
///
///				(C) 2004 Rob 'give me a job offer' Bateman 
///
//////////////////////////////////////////////////////////////////////////

#ifdef WIN32
	#ifndef NT_APP
		#define NT_APP
	#endif

	// crappy VC6 template warnings
	#pragma warning(disable:4786)
#endif


#include "CollisionSphere.h"
#include "Bone.h"
#include <maya/MPlug.h>
#include <maya/MFnDagNode.h>

///////////////////////////////////////////////////////////////////////////

/// \brief	This function outputs the Light to the specified file stream
/// \param	ofs	-	the output file stream
/// \param	obj	-	the handle to the maya node
/// 
void OutputCollisionSphere(std::ofstream& ofs,MObject &obj)
{
	MFnDagNode fnDep(obj);

	unsigned int iParent = BuildJointRef( fnDep.parent(0) );
	unsigned int iChild  = GetObjectID(fnDep.name().asChar());

	AddObjectToBone(iChild,iParent);


	float radius;

	MPlug plgRadius = fnDep.findPlug("radius");
	plgRadius.getValue(radius);


	ofs << "SPHERE "     << fnDep.name().asChar()
		<< "\n\tID "	 << iChild
		<< "\n\tParent " << iParent
		<< "\n\tRadius " << radius
		<< std::endl;
}


///////////////////////////////////////////////////////////////////////////

/// \brief	This function outputs the Light to the specified file stream
/// \param	ofs	-	the output file stream
/// \param	obj	-	the handle to the maya node
/// 
void OutputCollisionBox(std::ofstream& ofs,MObject &obj)
{
	MFnDagNode fnDep(obj);

	unsigned int iParent = BuildJointRef( fnDep.parent(0) );
	unsigned int iChild  = GetObjectID(fnDep.name().asChar());

	AddObjectToBone(iChild,iParent);


	float x,y,z;

	MPlug plgX = fnDep.findPlug("sizeX");
	MPlug plgY = fnDep.findPlug("sizeY");
	MPlug plgZ = fnDep.findPlug("sizeZ");

	plgX.getValue(x);
	plgY.getValue(y);
	plgZ.getValue(z);

	ofs << "BOX "     << fnDep.name().asChar()
		<< "\n\tID "	 << iChild
		<< "\n\tParent " << iParent
		<< "\n\tX " << x
		<< "\n\tY " << y
		<< "\n\tZ " << z
		<< std::endl;
}



///////////////////////////////////////////////////////////////////////////

/// \brief	This function outputs the Light to the specified file stream
/// \param	ofs	-	the output file stream
/// \param	obj	-	the handle to the maya node
/// 
void OutputCollisionCone(std::ofstream& ofs,MObject &obj)
{
	MFnDagNode fnDep(obj);

	unsigned int iParent = BuildJointRef( fnDep.parent(0) );
	unsigned int iChild  = GetObjectID(fnDep.name().asChar());

	AddObjectToBone(iChild,iParent);


	float angle,cap;

	MPlug plgAngle = fnDep.findPlug("ca");
	MPlug plgCap   = fnDep.findPlug("cap");

	plgAngle.getValue(angle);
	plgCap.getValue(cap);


	ofs << "CONE "     << fnDep.name().asChar()
		<< "\n\tID "	 << iChild
		<< "\n\tParent " << iParent
		<< "\n\tAngle " << angle
		<< "\n\tCap " << cap
		<< std::endl;
}




