//////////////////////////////////////////////////////////////////////////
/// 
/// \file	CollisionSphere.h
/// \author	Rob Bateman
///	\brief	In order to handle collisions for the character within the game,
///			I use render sphere nodes available in maya to roughly approximate
///			the character. Whilst these aren't ideal for the purpose (they
///			basically create a volume fog light thing) they are kinda the 
///			best blag for this purpose, and I actually can't be arsed writing
///			a custom plugin for the purpose.
/// 
///			The primary thing to understand is that ultimately there will 
///			a main sphere that surrounds the object that will be generated 
///			by code. This will be tested first, and if that test passes, 
///			then the spheres manually placed and parented to the hierarchy
///			will then be tested. 
///
/// \note	
/// 		This sourcefile is provided for free use for non-commercial purposes only. The
/// 		Software is provided 'as is' and no responsibility can be taken for any damages
/// 		that may be caused while using it. If you make use of this software within 
/// 		your code then it would be nice to have a small credit somewhere in your
/// 		finished product for my hard work :(
/// 		An e-mail saying you found it useful and possibly even a link to your product
/// 		would be really nice too ;)
///
///			Slip me a couple of quid and i might be kind enough to offer assistance in
/// 		sorting out your exporters ;) 
///
///				(C) 2004 Rob 'give me a job offer' Bateman 
/// 
//////////////////////////////////////////////////////////////////////////

#ifdef WIN32
	#pragma once
#endif

#ifndef COLLISION_SPHERE_H_INCLUDED_
#define COLLISION_SPHERE_H_INCLUDED_

	#ifdef WIN32
		#pragma warning(disable:4786)
	#endif

	#include <iostream>
	#include <maya/MObject.h>


	/// \brief	This function outputs the volume sphere to the specified file stream
	/// \param	ofs	-	the output file stream
	/// \param	obj	-	the handle to the maya node
	/// 
	extern void OutputCollisionSphere(std::ofstream& ofs,MObject &obj);


	/// \brief	This function outputs the volume box to the specified file stream
	/// \param	ofs	-	the output file stream
	/// \param	obj	-	the handle to the maya node
	/// 
	extern void OutputCollisionBox(std::ofstream& ofs,MObject &obj);


	/// \brief	This function outputs the volume cone to the specified file stream
	/// \param	ofs	-	the output file stream
	/// \param	obj	-	the handle to the maya node
	/// 
	extern void OutputCollisionCone(std::ofstream& ofs,MObject &obj);



#endif
