/////////////////////////////////////////////////////////////////////////////////////////
///
///	\file	Material.h
/// \author	Rob Bateman
/// \brief	This file defines the single function OutputMaterials which will output all
///			of the material information for phong,lambert and blinn shaders to the
///			output files. 
///
/// \note	
/// 		This sourcefile is provided for free use for non-commercial purposes only. The
/// 		Software is provided 'as is' and no responsibility can be taken for any damages
/// 		that may be caused while using it. If you make use of this software within 
/// 		your code then it would be nice to have a small credit somewhere in your
/// 		finished product for my hard work :(
/// 		An e-mail saying you found it useful and possibly even a link to your product
/// 		would be really nice too ;)
///
///			Slip me a couple of quid and i might be kind enough to offer assistance in
/// 		sorting out your exporters ;) 
///
///				(C) 2004 Rob 'give me a job offer' Bateman 
/// 
/////////////////////////////////////////////////////////////////////////////////////////

#ifdef WIN32
#pragma once
#endif

#ifndef MATERIALS_H_INCLUDED
#define MATERIALS_H_INCLUDED

#include <iostream>
#include <maya/MString.h>

/// \brief	This function outputs all of the material information in the scene
///			to the specified file stream.
/// \param	ofs	-	the output file stream
///
extern void OutputMaterials(std::ostream &ofs);

/// \brief	The materials are assigned a unique ID which is used to reference the
///			material that is used. This function will either returns a previously 
///			used ID or will generate a new ID if a new material is encountered.
///	\param	material	-	the name of the material to output 
/// \return	the ID number of the material.
/// 
extern unsigned int GetMaterialID(const MString &material);

#endif
