//////////////////////////////////////////////////////////////////////////////////////////////
/// \file	NurbsCurve.cpp
/// \author	Robert Bateman
/// \brief	Might as well add support for nurbs curves, not the quickest of rendererable
/// 		primitve, though can be used for motion paths, camera splines etc.
/// \note	
/// 		This sourcefile is provided for free use for non-commercial purposes only. The
/// 		Software is provided 'as is' and no responsibility can be taken for any damages
/// 		that may be caused while using it. If you make use of this software within 
/// 		your code then it would be nice to have a small credit somewhere in your
/// 		finished product for my hard work :(
/// 		An e-mail saying you found it useful and possibly even a link to your product
/// 		would be really nice too ;)
///
///			Slip me a couple of quid and i might be kind enough to offer assistance in
/// 		sorting out your exporters ;) 
///
///				(C) 2004 Rob 'give me a job offer' Bateman 
//////////////////////////////////////////////////////////////////////////////////////////////

#ifdef WIN32
#define NT_APP
#endif

#include "Bone.h"
#include "NurbsCurve.h"
#include <maya/MPoint.h>
#include <maya/MPointArray.h>


void OutputNurbsCurve(std::ostream &ofs,const MObject& obj)
{
	MFnNurbsCurve fnCurve(obj);

	unsigned int iParent	= BuildJointRef( fnCurve.parent(0) );
	unsigned int iNode		= GetObjectID( fnCurve.name().asChar() );	
	unsigned int Degree		= fnCurve.degree();
	unsigned int NumPoints	= fnCurve.numCVs();

	AddObjectToBone(iNode,iParent);

	// See if we have found a simple bezier curve
	if( Degree    == 3 && 
		NumPoints == 4 )
	{
		ofs << "BEZIER_CURVE "	<< fnCurve.name().asChar() 
			<< "\n\tID "		<< iNode
			<< "\n\tParentID "	<< iParent;

		MPointArray pnts;
		fnCurve.getCVs(pnts);

		for(unsigned int i=0;i<pnts.length();++i)
		{
			ofs << "\n\t\t" << (float)pnts[i].x << " " << (float)pnts[i].y << " " << (float)pnts[i].z;
		}

		ofs << "\n";
	}
	else
	// otherwise write as a bezier curve
	{
		ofs << "NURBS_CURVE "	<< fnCurve.name().asChar() 
			<< "\n\tID "		<< iNode
			<< "\n\tParentID "	<< iParent
			<< "\n\tDegree "	<< Degree
			<< "\n\tKnots "		<< (fnCurve.numKnots()+2) << "\n\t\t";
		unsigned int i;
		// write first knot (that maya ignores)
		if( (fnCurve.knot(1) - fnCurve.knot(0)) < 0.01f )
			ofs << (float)fnCurve.knot(0) << " ";
		else
			ofs << (float)( fnCurve.knot( 0 ) - 1 ) << " ";
		
		// write rest of the knots
		for( i=0;i<fnCurve.numKnots(); ++i )
			ofs << (float)fnCurve.knot(i) << " ";		
		
		// write last knot (that maya ignores)
		if( (fnCurve.knot(1) - fnCurve.knot(0)) < 0.01f )
			ofs << (float)( fnCurve.knot( fnCurve.numKnots()-1 ) ) << "\n";
		else
			ofs << (float)(fnCurve.knot( fnCurve.numKnots()-1 )+1) << "\n";
		
		
		ofs << "\tCVS " << fnCurve.numCVs() << "\n";
		MPointArray pnts;
		fnCurve.getCVs(pnts);
		
		for( i=0;i<pnts.length();++i)
		{
			ofs << "\n\t\t" << (float)pnts[i].x << " " << (float)pnts[i].y << " " << (float)pnts[i].z;
		}
		
		ofs << "\n";
	}
}



