//	
//	OK, now we are going to look at textures (note - not materials). Maya offers us
//	the really handy class MImage which allows us to load (though not save) any file format
//	supported by maya. This basically allows us to traverse the scene looking for file
//	texture nodes, read the filename, load the file and then output the data in our own
//	format. This may seem a bit long winded, however it basically gives us a HELL of a lot 
//	of freedom because we don't need to know how to load hectic file formats such as jpg
//	tiff etc. It also gives us an opportunity to check the texture size. 
//
//	OpenGL only supports textures that have dimension that is of power 2, ie 2,4,8,16,32 etc.
//	Now, if the texture file is not power of 2, we can simply resize it to a valid size and
//	print a warning. The constant g_DefaultTextureRes exists for this purpose. Currently
//	this is set to 128.
//
//	When it comes to storing this texture data we really don't want to store it in the same
//	file as the model data, mainly because there is FUCK loads of texture data, so it's better
//	to package all of the textures together into a single file. Basically to ensure that we
//	never loose textures or have problems with Full paths names when transferred to another 
//	computer, we will simply use an ID referencing system that will refer to a zero based 
//	index which represents the specific texture file.
//
//	err, bit pissed right now. Anyhow, For each image we want to store :
//
//	width	
//	height	
//	bpp		1/3/4
//
//	It's index is given by it's place in the file (starting at zero).
//
//	Now we could hold that data as 3 shorts, or 2 shorts and a bool. Thats a bit cack. 
//	I kinda like nice small fields, and a bit of data compaction. Instead of storing the
//	width in pixels, why not just store the power? With 4bits of data (a nibble) we can 
//	store a value between 0 and 15, ie 2^0 to 2^15, or 1 to 32768. So in 1byte we can store
//	the width and the height! If we then add 1byte to store the bpp we should have minimal
//	data use and a couple of bits left over (possibly compressed data flags?)	
//	
///
/// \note	
/// 		This sourcefile is provided for free use for non-commercial purposes only. The
/// 		Software is provided 'as is' and no responsibility can be taken for any damages
/// 		that may be caused while using it. If you make use of this software within 
/// 		your code then it would be nice to have a small credit somewhere in your
/// 		finished product for my hard work :(
/// 		An e-mail saying you found it useful and possibly even a link to your product
/// 		would be really nice too ;)
///
///			Slip me a couple of quid and i might be kind enough to offer assistance in
/// 		sorting out your exporters ;) 
///
///				(C) 2004 Rob 'give me a job offer' Bateman 

#ifdef WIN32
	#ifndef NT_APP
		#define NT_APP
	#endif
	
	#pragma warning(disable:4786)
#endif

// include each header for each class you need. Gets a bit laborious later on...
//
#include <maya/MImage.h>
#include <maya/MLibrary.h>
#include <maya/MGlobal.h>

#include <maya/MDagPath.h>
#include <maya/MDagPathArray.h>
#include <maya/MEulerRotation.h>
#include <maya/MFileIO.h>
#include <maya/MFloatMatrix.h>
#include <maya/MFloatVector.h>
#include <maya/MFloatVectorArray.h>
#include <maya/MMatrix.h>
#include <maya/MPlug.h>
#include <maya/MPoint.h>
#include <maya/MPointArray.h>

#include <maya/MFnAmbientLight.h>
#include <maya/MFnAreaLight.h>
#include <maya/MFnBlinnShader.h>
#include <maya/MFnDagNode.h>
#include <maya/MFnDirectionalLight.h>
#include <maya/MFnIkJoint.h>
#include <maya/MFnLambertShader.h>
#include <maya/MFnLight.h>
#include <maya/MFnMesh.h>
#include <maya/MFnPhongShader.h>
#include <maya/MFnPointLight.h>
#include <maya/MFnSkinCluster.h>
#include <maya/MFnSpotLight.h>
#include <maya/MFnTransform.h>

#include <maya/MItDependencyNodes.h>
#include <maya/MItMeshPolygon.h>
#include <maya/MItMeshVertex.h>

#include <algorithm>
#include <iostream>
#include <fstream>
#include <vector>
#include <string>
#include <map>

// link to some fairly funky maya libs. NOTE: The Image library only became part of maya
// from version 4.01 onwards. It is not available in version 4.0
//
#ifdef WIN32
#pragma comment(lib,"Foundation.lib")
#pragma comment(lib,"OpenMaya.lib")
#pragma comment(lib,"Image.lib")
#pragma comment(lib,"OpenMayaAnim.lib")
#endif

#include "Light.h"
#include "Material.h"
#include "NurbsSurface.h"
#include "NurbsCurve.h"
#include "Mesh.h"
#include "Texture.h"
#include "Bone.h"
#include "FileDialog.h"
#include "SkinCluster.h"
#include "CollisionSphere.h"

/// As a temporary hack, I am going to also build skeletons from joints that are attched 
/// to a skin cluster. The reason for this is straight forward. When we later deal with
/// skinning we will find that our skinned meshes are actually parented under the base
/// transform, not the transforms of the joints that are affecting it. This causes us
/// mild headaches because 
///
///	1. We do not want to export all transforms in the scene, if we did we would get a lot
///    of items such as camera nodes, texture placement transforms etc; that we really 
///	   dont want
/// 2. As yet we are only really exporting lights and meshes so it seems a bit pointless
///    to export transforms for objects we do not use. 
/// 
/// The method I have chosen to get around the problem is by using a solution where 
/// each object to be exported (mesh, light etc) adds it's parent to a bone export list.
/// The entire hierarchy that the particular bone is used in is then added into the 
/// export list also. 
///
/// This is where the skinning thing has a problem. If skinned meshes only have a single
/// transform, then none of the animated (inportant) bones will be exported. So, until 
/// I deal with skin-weights properly (Exporter8 / Loader8) This function will query the
/// bones used by a skin cluster and add those in also. Sorry if this seems a bit wierd....
///
void BuildSkinReference(const MObject& obj)
{
	MStatus s;
	MFnSkinCluster fnSkin(obj);
	
	MDagPathArray Joints;
	
	fnSkin.influenceObjects( Joints, &s );
	
	for( int i=0;i < Joints.length(); ++i )
	{
		if( Joints[i].node().hasFn(MFn::kTransform) )
			BuildJointRef( Joints[i].node() );
	}
}







#ifdef WIN32
int main(int argc,char** argv)
{
#else // hack for linux & gtk

int   g_argc=0;
char** g_argv=0;

int main(int argc,char** argv)
{
	g_argc=argc;
	g_argv=argv;
#endif

	// var to hold the input file name
	std::string infilename;

	// var to hold the output file name
	std::string outfilename;

	if (argc == 3 )
	{
		infilename  = argv[1];
		outfilename = argv[2];
	}
	else
	// if only one argument was specified, I'll assume that it is a maya binary
	// or ascii file, in which case use remove the 'mb' or 'ma' and add a 'mod' extension
	// to the end of the filename 
	// 
	if (argc == 2 )
	{
		char buffer[512]={0};
		if(!SaveFileDialog(buffer)) {
			return 0;
		}
		outfilename = buffer;
	}
	else
	if (argc == 1 )
	{
		char buffer[512]={0};
		if(!OpenFileDialog(buffer)) {
			return 0;
		}
		
		infilename = buffer;
		
		if(!SaveFileDialog(buffer)) {
			return 0;
		}
		
		// if outfile file was specified, use that
		//
		outfilename = buffer;
	}
	else {
		return 0;
	}


	std::string outFilePrefix;

	// if only one argument was specified, I'll assume that it is a maya binary
	// or ascii file, in which case use this - 'mb' or 'ma' and add a 'mod' extension
	// to the end of the filename 
	outFilePrefix = infilename;
	outFilePrefix.resize(outFilePrefix.size()-2);
	outFilePrefix += "tex";


	// initialise the maya library - This basically starts up Maya
	MLibrary::initialize(argv[0]);




	// open the file that the user specified 
	if( MS::kSuccess == MFileIO::open(infilename.c_str(),NULL,true) )
	{

		// do the texture file for a laugh....
		DoTextures(outFilePrefix.c_str());

		// create an output file stream
		std::ofstream ofs(outfilename.c_str());

		// make sure it opened OK
		if (!ofs) {
			std::cerr << "Could not open output file " << outfilename.c_str() << std::endl;
			MLibrary::cleanup();
			return 1;
		}

		// output material data
		OutputMaterials(ofs);


		// output the shapes 
		{
			// iterate through the mesh nodes in the scene
			MItDependencyNodes itDep(MFn::kInvalid);

			// we have to keep iterating until we get through all of the nodes in the scene
			while (!itDep.isDone()) 
			{
				// get the MObject hanle for this item in the list
				MObject item = itDep.item();

				switch(item.apiType()) 
				{
				// skin cluster node (holds skinning info)
				case MFn::kSkinClusterFilter:
					BuildSkinReference(item);
					AddSkinClusterRef(item);
					break;

				// rigid skinning info
				case MFn::kJointCluster:
					break;
					
				// a nurbs curve
				case MFn::kNurbsCurve:
					OutputNurbsCurve(ofs,item);
					break;
					
				// a nurbs surface node
				case MFn::kNurbsSurface:
					OutputNurbsSurface(ofs,item);
					break;

				// a basic non-visible sphere shape
				case MFn::kRenderSphere:
					OutputCollisionSphere(ofs,item);
					break;

				// a basic non-visible box shape
				case MFn::kRenderBox:
					OutputCollisionBox(ofs,item);
					break;

				// a basic non-visible cone shape
				case MFn::kRenderCone:
					OutputCollisionCone(ofs,item);
					break;

				// a locator node
				case MFn::kLocator: {
						MFnDagNode fnLoc(item);

						unsigned int iParent = BuildJointRef( fnLoc.parent(0) );
						unsigned int iChild  = GetObjectID( fnLoc.name().asChar() );

						ofs << "LOCATOR " << fnLoc.name().asChar() << "\n\tID "
							<< iChild << "\n\tParentID " << iParent << std::endl;
					}
					break;
				
				// a mesh
				case MFn::kMesh:
					OutputMesh(ofs,item);
					break;

				// a light shape
				case MFn::kLight:
				case MFn::kAreaLight:
				case MFn::kAmbientLight:
				case MFn::kDirectionalLight:
				case MFn::kPointLight:
				case MFn::kSpotLight:
					OutputLight(ofs,item);
					break;

				default:
					break;
				}

				// move to next node
				itDep.next();
			}
		}

		// output the transforms 
		// 
		OutputTransforms(ofs);
		OutputSkinClusters(ofs);
		OutputAnimation(ofs);

		ofs << "\n";

		// close the output file
		ofs.close();
	}
	else
	{
		std::cerr << "Could not open file " << infilename.c_str() << std::endl;
	}


	// cleanup maya data

	MLibrary::cleanup();

	return 0;
}

