///-----------------------------------------------------------------------------
/// 
/// \file	AnimCycle.h
/// \author	Rob Bateman
/// \brief	This file defines the XAnimCycle class. basicaly this class holds a 
///			set of animation cycles for each bone in the hierarchy. 
/// \note	
/// 		This sourcefile is provided for free use for non-commercial purposes only. The
/// 		Software is provided 'as is' and no responsibility can be taken for any damages
/// 		that may be caused while using it. If you make use of this software within 
/// 		your code then it would be nice to have a small credit somewhere in your
/// 		finished product for my hard work :(
/// 		An e-mail saying you found it useful and possibly even a link to your product
/// 		would be really nice too ;)
///
///			Slip me a couple of quid and i might be kind enough to offer assistance in
/// 		sorting out your exporters ;) 
///
///				(C) 2004 Rob 'give me a job offer' Bateman 
/// 
///-----------------------------------------------------------------------------

#ifdef WIN32
#pragma once
#pragma warning(disable:4786)
#endif

#ifndef ANIM_CYCLE_H_INCLUDED_
#define ANIM_CYCLE_H_INCLUDED_


#include <map>
#include "Bone.h"
#include "BoneAnimation.h"


///-----------------------------------------------------------------------------
/// \brief	This class holds the animation data for one bone/animation cycle.
///			At the moment this holds rotation,scale and translation keys. Later on 
///			I will show you how to reduce the amount of keyframes to reduce
///			the data storage requirements and also reduce the amount of calculation
///			required.
/// 
class XAnimCycle
	: protected std::map<unsigned int,XBoneAnimation*>
{
	
	friend std::istream& operator >> (std::istream& ifs,XAnimCycle &ac);

public:

	/// ctor
	XAnimCycle();
	
	/// dtor
	~XAnimCycle();

	/// \brief	This function updates the animation cycle by the specified amount
	/// \param	dt	-	the time increment
	/// 
	void Update(float dt);

	/// \brief	This function returns the requested bone animation
	/// \param	id	-	the ID of the bone animation to return
	/// \return	a pointer to the animation data
	///  
	XBoneAnimation* GetBoneAnim(unsigned int id) {
		iterator it = find(id);
		if( it == end() ) {
			return NULL;
		}
		return it->second;
	}

	/// \brief	This function returns the current time of the animation cycle
	/// \return	a reference to the internal time info structure.
	///
	XTimeInfo& GetTime() {
		return m_Time;
	}

protected:

	/// the name of the animation cycle.
	std::string m_AnimName;

	/// internally the animcycle class holds timing info for this cycle
	XTimeInfo m_Time;
};

#endif

