//-----------------------------------------------------------------------
///	\file    	BezierCurve.h
///	\date   	2/5/2004   12:31
///	\author		Rob Bateman
///	\brief      
/// \note       
///
//-----------------------------------------------------------------------

#ifndef __BezierCurve__H__
#define __BezierCurve__H__

#ifdef WIN32
	#define WIN32_LEAN_AND_MEAN
	#include <Windows.h>
#endif
#include <GL/gl.h>

#include <vector>
#include <fstream>
#include <Vector.h>
#include <Transform.h>
#include <Plane.h>

#include "Material.h"
#include "Object.h"
#include "ParametricCurve.h"

//-----------------------------------------------------------------------
///	\brief  A pretty cacky simple class to hold a cubic bezier curve	
///
class XBezierCurve
	: public XParametricCurve
{
	friend std::istream &operator>>(std::istream& ifs,XBezierCurve& nc);
public:

	/// \ctor
	XBezierCurve(void);
	
	/// \dtor
	~XBezierCurve(void);



	//-------------------------------------------------------------------
	/// \brief	gets the type of the object
	/// \return	The type of the object XFn::BezierCurve
	///
	virtual XFn::Type GetType() const {
		return XFn::BezierCurve;
	}


private:

	//-------------------------------------------------------------------
	/// \brief	This is overidden to update the level of detail on the 
	///			parametric shape
	/// 
	virtual void OnSetLod();

	//-------------------------------------------------------------------
	///	\brief  Evaluates the current surface positions and normals
	///
	virtual void Evaluate(XVertList*);

	//-------------------------------------------------------------------
	///	\brief  This function evaluates the vertex positions
	///	
	void GenerateVertices();

	/// a set of pre-cached blending functions for the bezier curve
	std::vector<float> m_BlendingFunctions; 



};


#endif


