//-------------------------------------------------------------------
///	\file    	BezierPatch.h
///	\date   	2/5/2004   22:24
///	\author		Rob Bateman
///	\brief      
/// \note       
///
//-------------------------------------------------------------------

#ifndef __BezierPatch__H__
#define __BezierPatch__H__

#ifdef WIN32
	#pragma once
#endif

#include "Object.h"
#include "ParametricSurface.h"

class XBezierPatch
	: public XParametricSurface
{

	friend std::istream &operator>>(std::istream&,XBezierPatch&);
public:

	/// \brief	ctor
	XBezierPatch();


	/// \brief	dtor
	~XBezierPatch();


	/// gets the type of the object
	virtual XFn::Type GetType() const {
		return XFn::BezierPatch;
	}

protected:

	//------------------------------------------------------------------- Parametric Interface functions

	//-------------------------------------------------------------------
	/// \brief	This is overidden to update the level of detail on the 
	///			parametric shape
	/// 
	virtual void OnSetLod();

	//-------------------------------------------------------------------
	///	\brief  Evaluates the current surface positions and normals
	///
	virtual void Evaluate(XVertList*);

private:

	//------------------------------------------------------------------- Hard working functions !

	//-------------------------------------------------------------------
	///	\brief  This function evaluates the vertex positions
	///	
	void GenerateVertices();


	//-------------------------------------------------------------------
	///	\brief  This function evaluates the vertex positions for the 
	///			intermediate surface curves.
	///	\param	UseVerts	-	an array of vertices to use for the 
	///							control vertices.
	/// 
	void GenerateTemporaryVertices(XVertList*);



	/// num blend funcs = 4 * GetLOD()
	/// 
	std::vector<float> m_BlendFuncs;

	/// an array of temporary vertices for the V calculation.
	/// The first stage is to interpolate across the U direction,
	/// the results of which are then stored in the following
	/// array for the surface evaluation in the other direction.
	/// 
	///	In essence this will hold a set of curves across the 
	/// U direction.
	/// 
	std::vector<XVector> m_TempPoints;


};

#endif
