///-----------------------------------------------------------------------------
/// 
/// \file	BoneAnimation.cpp
/// \author	Rob Bateman
/// \brief	This file defines the class XBoneAnimation which holds the animation 
///			data for one bone / animation cycle. Later on I will replace this 
///			with an animation mixer to blend different animation cycles 
///			together and also allows for lower/upper body seperation.
/// \note	
/// 		This sourcefile is provided for free use for non-commercial purposes only. The
/// 		Software is provided 'as is' and no responsibility can be taken for any damages
/// 		that may be caused while using it. If you make use of this software within 
/// 		your code then it would be nice to have a small credit somewhere in your
/// 		finished product for my hard work :(
/// 		An e-mail saying you found it useful and possibly even a link to your product
/// 		would be really nice too ;)
///
///			Slip me a couple of quid and i might be kind enough to offer assistance in
/// 		sorting out your exporters ;) 
///
///				(C) 2004 Rob 'give me a job offer' Bateman 
/// 
///-----------------------------------------------------------------------------

#include "BoneAnimation.h"

#include <assert.h>

XBoneAnimation::XBoneAnimation(void) 
{
}

XBoneAnimation::~XBoneAnimation(void)
{
}

/// this func updates the current anim cycle
void XBoneAnimation::Update(OUT XTransform& xform,const XTimeInfo& time) 
{
	// make sure we have keyframes on the anim chunk
	assert( m_Keys.size() );

	// calculate the index into the bone anim data array
	int index = time.GetFrameNum();

	// index to last keyframe
	int sz = static_cast<int>(m_Keys.size()-1);

	if(index > sz || sz < 0) {}
	else
	// make sure we have 2 keys to interpolate between
	if( index < sz ) 
	{
		// interpolate the keyframes
		LERP( xform, time.GetFrameT(), m_Keys[index], m_Keys[index+1] );
	}
	else 
	{
		// set to end keyframe
		xform  = m_Keys[sz];
	}
}

/// stream extraction operator
std::istream& operator >> (std::istream& ifs,XBoneAnimation &ba) 
{
	// loop through all of the keyframes
	for(unsigned int i=0;i<static_cast<unsigned int>(ba.m_Keys.size());++i) 
	{
		// read rotation translation and scale keys
		ifs >> ba.m_Keys[i];
	}
	return ifs;
}

/// stream extraction operator
std::istream& operator >> (std::istream& ifs,XTransform &xf)
{
	// read rotation translation and scale
	return ifs >> xf.m_Rotation >> xf.m_Translation >> xf.m_Scale;
}
