///-----------------------------------------------------------------------------
/// 
/// \file	Light.h
/// \author	Rob Bateman
/// \brief	This file defines the class XLight whixh is used to hold information
///			about a light. 
/// \note	
/// 		This sourcefile is provided for free use for non-commercial purposes only. The
/// 		Software is provided 'as is' and no responsibility can be taken for any damages
/// 		that may be caused while using it. If you make use of this software within 
/// 		your code then it would be nice to have a small credit somewhere in your
/// 		finished product for my hard work :(
/// 		An e-mail saying you found it useful and possibly even a link to your product
/// 		would be really nice too ;)
///
///			Slip me a couple of quid and i might be kind enough to offer assistance in
/// 		sorting out your exporters ;) 
///
///				(C) 2004 Rob 'give me a job offer' Bateman 
/// 
///-----------------------------------------------------------------------------

#include "Light.h"
#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <Windows.h>
#endif
#include <GL/gl.h>


XLight::XLight(unsigned int id)
	: m_LightNum(id)
{
	m_Color[0] = 0;
	m_Color[1] = 0;
	m_Color[2] = 0;
	m_Color[3] = 0;
	m_Position[0] = 0;
	m_Position[1] = 0;
	m_Position[2] = 7;
	m_Position[3] = 1;
}

XLight::~XLight(void)
{
}

float g_Amb[4] = {0.3f,0.3f,0.3f,1.0f};
float g_Spc[4] = {1.0f,1.0f,1.0f,1.0f};

/// this function draws the mesh data
void XLight::Enable()
{
	/// set the ambient colour
	glLightfv(GL_LIGHT0 + m_LightNum,GL_AMBIENT,g_Amb);

	/// set the diffuse colour
	glLightfv(GL_LIGHT0 + m_LightNum,GL_DIFFUSE,m_Color);

	/// set the specular colour
	glLightfv(GL_LIGHT0 + m_LightNum,GL_SPECULAR,g_Spc);
	
	/// set the lights position
	glLightfv(GL_LIGHT0 + m_LightNum,GL_POSITION,m_Position);

	/// enable the light
	glEnable(GL_LIGHT0 + m_LightNum);
}

/// this function draws the mesh data
void XLight::Disable()
{
	glDisable(GL_LIGHT0 + m_LightNum);
}


std::istream &operator>>(std::istream& ifs,XLight& l) {
	char buffer[64];
	ifs >> buffer; // name
	ifs >> buffer; // "id"
	ifs >> l.m_ID;
	ifs >> buffer  // "Parent="
		>> l.m_ParentID;
	ifs >> buffer; // "Type="
	ifs >> buffer; // kPointLight, kAmbientLight, kSpotLight etc
	ifs >> buffer; // "Color="

	ifs >> l.m_Color[0] >> l.m_Color[1] >> l.m_Color[2];

	ifs >> buffer; // "Intensity="

	ifs >> l.m_fIntensity;


	return ifs;
}
