///-----------------------------------------------------------------------------
/// 
/// \file	Light.h
/// \author	Rob Bateman
/// \brief	This file defines the class XLight whixh is used to hold information
///			about a light. 
/// \note	
/// 		This sourcefile is provided for free use for non-commercial purposes only. The
/// 		Software is provided 'as is' and no responsibility can be taken for any damages
/// 		that may be caused while using it. If you make use of this software within 
/// 		your code then it would be nice to have a small credit somewhere in your
/// 		finished product for my hard work :(
/// 		An e-mail saying you found it useful and possibly even a link to your product
/// 		would be really nice too ;)
///
///			Slip me a couple of quid and i might be kind enough to offer assistance in
/// 		sorting out your exporters ;) 
///
///				(C) 2004 Rob 'give me a job offer' Bateman 
/// 
///-----------------------------------------------------------------------------


#ifdef WIN32
#pragma once
#endif

#ifndef __XLIGHT__H_
#define __XLIGHT__H_
#include <iostream>

#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <Windows.h>
#endif
#include <GL/gl.h>

#include "./Object.h"

class XLight : public XObject
{
	friend std::istream &operator>>(std::istream&,XLight&);

public:


	/// ctor
	XLight(unsigned int id);

	/// dtor
	~XLight(void);

	/// this function draws the mesh data
	void Enable();

	/// this function draws the mesh data
	void Disable();

	void UpdatePosition(float x,float y,float z) {
		m_Position[0] = x;
		m_Position[1] = y;
		m_Position[2] = z;
	}	

	/// gets the type of the object
	virtual XFn::Type GetType() const {
		return XFn::Light;
	}

protected:
	
	/// openGl supports up to eight lights.
	unsigned int m_LightNum;

	/// the intensity of the light
	float m_fIntensity;

	/// the lights colour
	float m_Color[4];

	/// the position of the light
	float m_Position[4];
};

#endif


