///-----------------------------------------------------------------------------
/// 
/// \file	Material.cpp
/// \author	Rob Bateman
/// \brief	I've tried to make this loader a bit more generic. The material 
///			class is now able to store colour channels, ie a colour and a texture
///			for ambient, diffuse, specular etc. With basic openGL this isn't of 
///			much use because you can only use 1 texture 
///			(though 2 with mutlitexturing extension). This is mainly done for my
///			own purposes cos I use CG shaders so a bit more flexibility is nice.
/// 
///			At the moment, if a texture is assigned to the diffuse channel, then 
///			it is used as the surface texture.
/// \note	
/// 		This sourcefile is provided for free use for non-commercial purposes only. The
/// 		Software is provided 'as is' and no responsibility can be taken for any damages
/// 		that may be caused while using it. If you make use of this software within 
/// 		your code then it would be nice to have a small credit somewhere in your
/// 		finished product for my hard work :(
/// 		An e-mail saying you found it useful and possibly even a link to your product
/// 		would be really nice too ;)
///
///			Slip me a couple of quid and i might be kind enough to offer assistance in
/// 		sorting out your exporters ;) 
///
///				(C) 2004 Rob 'give me a job offer' Bateman 
/// 
///-----------------------------------------------------------------------------

#include "Material.h"
#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <Windows.h>
#endif
#include <GL/gl.h>

XMaterial::XMaterial(void)
{
}

XMaterial::~XMaterial(void)
{
}

/// as an optimisation, we will track which material is currently in use. This will 
/// try to minimise the state changes within OpenGL
XMaterial* XMaterial::ms_CurrentMaterial = NULL;


void XMaterial::gl(){
	// make sure we don't bind the material more than nessecary
	if (this==ms_CurrentMaterial) {
		return;
	}
	ms_CurrentMaterial = this;

	if (m_Diffuse.m_TextureObject!=0) {
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D,m_Diffuse.m_TextureObject);
	}
	else {
		glDisable(GL_TEXTURE_2D);
	}

	glMaterialfv(GL_FRONT,GL_AMBIENT,m_Ambient.m_Colour);
	glMaterialfv(GL_FRONT,GL_DIFFUSE,m_Diffuse.m_Colour);
	glMaterialfv(GL_FRONT,GL_SPECULAR,m_Specular.m_Colour);
	glMaterialfv(GL_FRONT,GL_EMISSION,m_Emission.m_Colour);
	glMaterialf(GL_FRONT,GL_SHININESS,m_Shininess);
}

	
std::istream& operator >> (std::istream& ifs,XPhong& ph)
{
	char buffer[256];

	ifs >> buffer // name
		>> ph.m_Ambient
		>> ph.m_Diffuse
		>> ph.m_Specular
		>> ph.m_Emission
		>> ph.m_Transparency
		>> buffer // shininess
		>> ph.m_Shininess;
		return ifs;
}

std::istream& operator >> (std::istream& ifs,XLambert& ph)
{
	char buffer[256];

	ifs >> buffer // name
		>> ph.m_Ambient
		>> ph.m_Diffuse
		>> ph.m_Emission
		>> ph.m_Transparency;

		return ifs;
}

std::istream& operator >> (std::istream& ifs,XBlinn& ph)
{
	char buffer[256];

	ifs >> buffer // name
		>> ph.m_Ambient
		>> ph.m_Diffuse
		>> ph.m_Specular
		>> ph.m_Emission
		>> ph.m_Transparency
		>> buffer // shininess
		>> ph.m_Shininess;
	float f;
	ifs >> buffer >> f;
		return ifs;
}
