///-----------------------------------------------------------------------------
/// 
/// \file	Mesh.cpp
/// \author	Rob Bateman
/// \brief	a simple file loader for a simple file... This loader basically 
///			uses streaming operators to read and write the file. The data is 
///			stored in a set os basic classes, XVertex, XNormal, XTexCoord and
///			XFace.
/// 
///			The XMesh class then holds the data within arrays, these array types 
///			are typedefs of std::vector<XVertex> etc.
/// \note	
/// 		This sourcefile is provided for free use for non-commercial purposes only. The
/// 		Software is provided 'as is' and no responsibility can be taken for any damages
/// 		that may be caused while using it. If you make use of this software within 
/// 		your code then it would be nice to have a small credit somewhere in your
/// 		finished product for my hard work :(
/// 		An e-mail saying you found it useful and possibly even a link to your product
/// 		would be really nice too ;)
///
///			Slip me a couple of quid and i might be kind enough to offer assistance in
/// 		sorting out your exporters ;) 
///
///				(C) 2004 Rob 'give me a job offer' Bateman 
/// 
///-----------------------------------------------------------------------------
#include "Mesh.h"
#include <assert.h>

//////////////////////////////////////////////////////////////////////////	XTexCoord

std::istream &operator>>(std::istream& ifs,XTexCoord& v)	{
	return ifs >> v.u >> v.v;
}

//////////////////////////////////////////////////////////////////////////	XFace

/// \brief	This function sets the number of indices within a face
/// \param	numPoints	-	the number of points within the face
/// 
void XFace::SetSize(unsigned int numPoints) 
{
	// ensure a minimum of 3 points
	if(numPoints<3)
		numPoints = 3;

	// create array of points
	XPointIndex* temp = new XPointIndex[numPoints];

	// copy over old data. Amount to copy depends on whether
	// array has shrunk or increased
	if(m_Points) {
		if (m_NumPoints<numPoints) {
			memcpy(temp,m_Points,sizeof(XVertex)*m_NumPoints);
		} else {
			memcpy(temp,m_Points,sizeof(XVertex)*numPoints);
		}

	}
	m_NumPoints = numPoints;

	// delete old data
	delete [] m_Points;

	// store reference to allocated memory
	m_Points = temp;
}

bool bHasUvs=true;
std::istream &operator>>(std::istream& ifs,XPointIndex& pi) {
	ifs >> pi.v >> pi.n ;
	if(bHasUvs)
		ifs >> pi.t;
	return ifs;
}

/// srteaming operator for the face
std::istream &operator>>(std::istream& ifs,XFace& f)	
{
	char buffer[10];
	int vcount;
	ifs >> buffer >> vcount;
	f.SetSize(vcount);

	// read material index for face
	ifs >> buffer >> f.m_Material;

	XPointIndex *p = f.m_Points;
	XPointIndex *end = p + vcount;
	for(;p!=end;++p) 
	{
		ifs >> *p;
	}
	return ifs;
}


//////////////////////////////////////////////////////////////////////////	XMesh

/// \brief	this function draws all the faces in the mesh
///
void XMesh::gl(XMaterialList& mats,
			   XVertList* UseVerts,
			   XNormList* UseNorms) {
	
	if( !DrawMeshes() )
		return;

	XFaceIt it = m_Faces.begin();
	for( ; it != m_Faces.end() ; ++it ) 
	{
		// make sure we have points
		assert(it->m_Points);

		// ensure a valid material ID
		assert( it->m_Material <= mats.size() );

		if(it->m_Material !=0)
			// Bind the relevant material
			mats[it->m_Material-1]->gl();

		XPointIndex *p = it->m_Points;
		XPointIndex *end = p + it->m_NumPoints;

		glBegin(GL_POLYGON);
		for(;p!=end;++p) 
		{
			if(m_TexCoords.size())
				m_TexCoords[p->t].gl();
			(*UseNorms)[p->n].gl();
			(*UseVerts)[p->v].gl();
		}
		glEnd();
	}
}

/// \brief	stream insertion operator for the mesh class
/// 
std::istream &operator>>(std::istream& ifs,XMesh& m)
{
	char buffer[256];
	int temp;

	// read mesh name
	ifs >> buffer;
	m.m_Name = buffer;
	ifs >> buffer;
	ifs >> m.m_ID;
	ifs >> buffer;
	ifs >> m.m_ParentID;

	// read vertex count
	ifs >> buffer >> temp;
	m.m_Vertices.resize(temp);

	// read normal count
	ifs >> buffer >> temp;
	m.m_Normals.resize(temp);

	// read tex coord count
	ifs >> buffer >> temp;
	m.m_TexCoords.resize(temp);

	// read face count
	ifs >> buffer >> temp;
	m.m_Faces.resize(temp);

	std::cout << "1" << std::endl;
	// read vertices
	ifs >> buffer;
	{
		for( XVertIt it = m.m_Vertices.begin(); it != m.m_Vertices.end() ; ++it ) 
			ifs >> *it;	
	}
	std::cout << "2" << std::endl;
	// read normals
	ifs >> buffer;
	{
		for( XNormIt it = m.m_Normals.begin(); it != m.m_Normals.end() ; ++it ) 
			ifs >> *it;
	}
	std::cout << "3" << std::endl;

	if( m.m_TexCoords.size() > 0)
	// read texture coords
	{
		bHasUvs=true;

		ifs >> buffer;
		for( XTexIt it = m.m_TexCoords.begin(); it != m.m_TexCoords.end() ; ++it ) 
			ifs >> *it;
	}
	else bHasUvs=false;
	std::cout << "4" << std::endl;

	// read the faces
	{
		for( XFaceIt it = m.m_Faces.begin(); it != m.m_Faces.end() ; ++it ) 
			ifs >> *it;
	}
	std::cout << "5" << std::endl;

	return ifs;
}

