///-----------------------------------------------------------------------------
/// 
/// \file	NurbsCurve.h
/// \author	Rob Bateman
/// \brief	defines the structures to hold a NurbsCurve. The curve is evaluated
/// 		using the Cox-De-Boor algorithm.
/// \note	
/// 		This sourcefile is provided for free use for non-commercial purposes only. The
/// 		Software is provided 'as is' and no responsibility can be taken for any damages
/// 		that may be caused while using it. If you make use of this software within 
/// 		your code then it would be nice to have a small credit somewhere in your
/// 		finished product for my hard work :(
/// 		An e-mail saying you found it useful and possibly even a link to your product
/// 		would be really nice too ;)
///
///			Slip me a couple of quid and i might be kind enough to offer assistance in
/// 		sorting out your exporters ;) 
///
///				(C) 2004 Rob 'give me a job offer' Bateman 
/// 
///-----------------------------------------------------------------------------


//////////////////////////////////////////////////////////////////////////

#ifdef WIN32
#pragma once
#endif


#ifndef XNURBS_CURVE__H_
#define XNURBS_CURVE__H_

//////////////////////////////////////////////////////////////////////////
/// 
#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <Windows.h>
#endif
#include <GL/gl.h>

#include <vector>
#include <fstream>
#include <Vector.h>
#include <Transform.h>
#include <Plane.h>

#include "Material.h"
#include "Object.h"
#include "ParametricCurve.h"



class XNurbsCurve 
	: public XParametricCurve
{
	friend std::istream &operator>>(std::istream&,XNurbsCurve&);
public:

	XNurbsCurve() ;
	~XNurbsCurve() ;

	/// \brief	returns The degree of the curve
	/// \return	the curve degree
	short Degree() const {
		return m_Degree;
	}
	
	/// gets the type of the object
	virtual XFn::Type GetType() const {
		return XFn::NurbsCurve;
	}
	
private:

	/// \brief	This function is called by the 
	void OnSetLod();

	virtual void Evaluate(XVertList* UseVerts);
	
	/// num blend funcs = numCvs * levelOfDetail
	std::vector<float> m_BlendFuncs;
	
	/// the curve degree in the U direction
	short m_Degree;
	
	/// the knot vector in U direction
	std::vector<float> m_Knots;

};


#endif


