//-------------------------------------------------------------------
///	\file    	Options.h
///	\date   	2/5/2004   3:04
///	\author		Rob Bateman
///	\brief      Defines a basic set of viewing options for the 
///				scene.
/// \note       
///
//-------------------------------------------------------------------

#ifndef __Options__H__
#define __Options__H__

struct XOptions
{
	//-------------------------------------------------------------------
	
	/// toggles Nurbs surface visibility on and off
	void ToggleNurbs() const { m_bDrawNurbs = !m_bDrawNurbs; }

	/// toggles Nurbs curve visibility on and off
	void ToggleCurves() const { m_bDrawCurves = !m_bDrawCurves; }

	/// toggles mesh visibility on and off
	void ToggleMeshes() const { m_bDrawMeshes = !m_bDrawMeshes; }

	/// toggles bone hierarchy visibility on and off
	void ToggleJoints() const { m_bDrawJoints = !m_bDrawJoints; }

	/// toggles collision object visibility on and off
	void ToggleCollisionObjects() const { m_bCollisionObjects = !m_bCollisionObjects; }

	/// toggles locator node visibility on and off
	void ToggleLocators() const { m_bDrawLocators = !m_bDrawLocators; }

	/// toggles wireframe display
	void ToggleWireframe() const { m_bWireFrame = !m_bWireFrame; }

	/// toggles textured display
	void ToggleTextured() const { m_bTextured = !m_bTextured; }

	/// toggles back face culling
	void ToggleCulling() const { m_bCulling = !m_bCulling; }

	/// toggles display of the grid
	void ToggleGrid() const { m_bGrid = !m_bGrid; }

	/// toggles the visibility of the axes
	void ToggleAxes() const { m_bAxes = !m_bAxes; }

	/// toggles the visibility of the axes
	void ToggleSeperateSpecular() const { m_bSS = !m_bSS; }


	//-------------------------------------------------------------------

	/// returns true if we want to see the axes
	const bool ViewAxes() const { return m_bAxes; }

	/// returns true if we want to see the grid
	const bool ViewGrid() const { return m_bGrid; }

	/// returns true if we are to draw Nurbs Surfaces
	const bool DrawNurbs() const { return m_bDrawNurbs; }

	/// returns true if we are to draw nurbs curves
	const bool DrawCurves() const { return m_bDrawCurves; }

	/// returns true if we are to draw meshes
	const bool DrawMeshes() const { return m_bDrawMeshes; }

	/// returns true if we are to draw joints
	const bool DrawJoints() const { return m_bDrawJoints; }

	/// returns true if we are to draw collision objects
	const bool DrawSpheres() const { return m_bCollisionObjects; }

	/// returns true if we are to draw locator nodes
	const bool DrawLocators() const { return m_bDrawLocators; }

	/// returns true if we are to draw in wire frame 
	const bool IsWireframe() const { return m_bWireFrame; }
	
	/// returns true if the model wants texturing...
	const bool IsTextured() const { return m_bTextured; }
	
	/// returns true if back face culling is enabled
	const bool IsCulled() const { return m_bCulling; }

	//-------------------------------------------------------------------

	/// returns the current scale of locator nodes
	const float LocatorScale() const { return m_fLocatorScale; }

	//-------------------------------------------------------------------

	/// sets the current locator node scale
	void SetLocatorScale(float f) const { m_fLocatorScale = f; }

	bool SS() const { return m_bSS; }


private:

	/// the scale of the locators
	static float m_fLocatorScale;
	/// should we draw nurbs surfaces?
	static bool m_bDrawNurbs;
	/// should we draw nurbs curves?
	static bool m_bDrawCurves;
	/// should we draw mesh data?
	static bool m_bDrawMeshes;
	/// should we draw transforms?
	static bool m_bDrawJoints;
	/// should we draw spheres ?
	static bool m_bCollisionObjects;
	/// should we draw locator nodes?
	static bool m_bDrawLocators;
	/// should we draw in wireframe?
	static bool m_bWireFrame;
	/// should we draw textures on objects that have them?
	static bool m_bTextured;
	/// should we draw textures on objects that have them?
	static bool m_bCulling;
	/// do you want to see the axes?
	static bool m_bAxes;
	/// do you want to see the grid?
	static bool m_bGrid;
	/// use seperate specular?
	static bool m_bSS;
};

#endif
