#include "ParametricSurface.h"


XParametricSurface::XParametricSurface() 
	: XObject(),
	  XParametricBase(this),
	  m_VertexArray(0),
	  m_NormalArray(0),
	  m_UvArray(0),
	  m_IndexArray(0),
	  m_NumCvsU(0),
	  m_NumCvsV(0)
{
}

XParametricSurface::~XParametricSurface()
{ 
	delete [] m_VertexArray;
	delete [] m_NormalArray;
	delete [] m_UvArray;
	delete [] m_IndexArray;

}

//-------------------------------------------------------------------
/// this function draws the surface data
void XParametricSurface::gl(XMaterialList& mats,
					   XVertList* UseVerts,
					   XNormList* UseNorms)
{

	if( !DrawNurbs() )
		return;

	// If you wanted the surface to re-calculate it's blending
	// functions every frame, you could theoretically un-comment
	// this call to do it. It would however be better to use lazy
	// evaluation and only change the blending factors when the 
	// Lod changes

	//	OnSetLod();


	// The parametric base can be 'locked' in place to prevent pointless
	// calculation of the surface if it's not needed.
	//
	if( !Locked() ) {
		Evaluate(UseVerts);
	}

	if (m_Material != 0) {
		mats[m_Material-1]->gl();
	}

	// enable vertex arrays
	glVertexPointer(3,GL_FLOAT,0,m_VertexArray);
	glNormalPointer(GL_FLOAT,0,m_NormalArray);
	glTexCoordPointer(2,GL_FLOAT,0,m_UvArray);

	glEnableClientState( GL_VERTEX_ARRAY );
	glEnableClientState( GL_NORMAL_ARRAY );
	glEnableClientState( GL_TEXTURE_COORD_ARRAY );

	unsigned int *ptr = m_IndexArray;
	unsigned int *end = ptr + (2*GetLOD()*(GetLOD()-1));
	unsigned int sz = 2*GetLOD();
	for ( ; ptr != end; ptr += sz )
	{
		glDrawElements(GL_TRIANGLE_STRIP,sz,GL_UNSIGNED_INT,ptr);
	}
	glDisableClientState( GL_VERTEX_ARRAY );
	glDisableClientState( GL_NORMAL_ARRAY );
	glDisableClientState( GL_TEXTURE_COORD_ARRAY );
}


//-------------------------------------------------------------------
///	\brief  This function evaluates the normal vectors
///	
void XParametricSurface::GenerateNormals()
{
	// pre-calculate some useful values
	unsigned int lod_minus_one=GetLOD()-1;
	unsigned int next_line_offset=3*GetLOD();
	unsigned int i,j;


	// Loop through all points in the triangle strip excluding far
	// rows. 
	for (i=0;i!=lod_minus_one;++i)
	{ 
		float *pPoint = m_VertexArray + (i * next_line_offset);
		float *pNorm  = m_NormalArray + (i * next_line_offset);

		for (j=0;j!=lod_minus_one;++j,pPoint+=3)
		{
			float* pNextPoint = pPoint + 3;
			float* pNextLine  = pPoint + next_line_offset;

			XPlane PolyFace = XPlane( XVector(pNextLine), XVector(pNextPoint), XVector(pPoint)  );

			XVector Norm ;

			Norm = PolyFace.Normal();

			*pNorm = Norm.x;
			++pNorm;
			*pNorm = Norm.y;
			++pNorm;
			*pNorm = Norm.z;
			++pNorm;
		}
		if( m_TypeV == 2 )
		{
			for(int k=0;k<3;++k,++pNorm)
				*pNorm = m_NormalArray[i*next_line_offset + k];
		}
		else {
			for(int k=0;k<3;++k,++pNorm)
				*pNorm = *(pNorm-3);
		}
	}
	float *pNorm1 = m_NormalArray + (lod_minus_one * next_line_offset);
	float *pNorm2;
	
	if (m_TypeV == 2) {
		pNorm2 = m_NormalArray;
	}
	else {
		pNorm2 = m_NormalArray + ( (lod_minus_one-1) * next_line_offset);
	}
	for (i=0;i!=next_line_offset;++i)
	{ 
		*pNorm1 = *pNorm2;
		++pNorm2;
		++pNorm1;
	}
}

//-------------------------------------------------------------------
///	\brief  This function evaluates the texture coords
///	
void XParametricSurface::GenerateTexCoords()
{

	// change to 1 to toggle the texture generation between, 
	//
	//	0 - per surface
	//	1 - per knot
	//
#if 0		// using knot values for tex coords

	const float u_start = m_KnotsU[m_DegreeU];
	const float u_end   = m_KnotsU[m_KnotsU.size()-1-m_DegreeU];
	const float u_total = u_end - u_start;
	const float u_inc   = u_total/(GetLOD()-1);

	const float v_start = m_KnotsV[m_DegreeV];
	const float v_end   = m_KnotsV[m_KnotsV.size()-1-m_DegreeV];
	const float v_total = v_end - v_start;
	const float v_inc   = v_total/(GetLOD()-1);

#else		// using zero to one range for texture

	const float u_start = 0;
	const float u_end   = 1;
	const float u_total = u_end - u_start;
	const float u_inc   = u_total/(GetLOD()-1);

	const float v_start = 0;
	const float v_end   = 1;
	const float v_total = v_end - v_start;
	const float v_inc   = v_total/(GetLOD()-1);

#endif

	float u    = u_start;
	float *ptr = m_UvArray;

	for(int i=0;i<GetLOD();++i)
	{
		float v=v_start;

		for(int j=0;j<GetLOD();++j)
		{
			v += v_inc;

			// assign texture coord into array
			*ptr = u;
			++ptr;
			*ptr = v;
			++ptr;
		}
		u += u_inc;
	}

}

//-------------------------------------------------------------------
///	\brief  This function evaluates the vertex indices for the triangle
///			arrays.
///	
void XParametricSurface::GenerateIndices()
{
	int maxIndex = 2*GetLOD()*(GetLOD()-1);
	int step = GetLOD();
	unsigned int* ptr = m_IndexArray;

	for (int i = 0;i!=GetLOD()*(GetLOD()-1) ;++i)
	{
		*ptr = i;
		++ptr;
		*ptr = i+step;
		++ptr;
	}
}

