//-------------------------------------------------------------------
///	\file    	ParametricSurface.h
///	\date   	2/5/2004   20:13
///	\author		Rob Bateman
///	\brief      
/// \note       
///
//-------------------------------------------------------------------

#ifndef __ParametricSurface__H__
#define __ParametricSurface__H__

#ifdef WIN32
#pragma once
#endif

#include "Object.h"
#include "ParametricBase.h"


//-------------------------------------------------------------------
/// \brief	This class forms the common base class of all parametric
///			(and or dynamic) surface types. It contains common 
///			functionality to generate normals,indicies,texture coords,
///			and then render them as triangle strips. Derived classes
///			have to then supply the methods for calculating the vertex
///			coordinates for the surface and re-allocating memory when
///			the LOD changes.
///
class XParametricSurface
	: public XObject,
	  public XParametricBase
{
public:

	/// ctor
	XParametricSurface();

	/// dtor
	~XParametricSurface();


	//-------------------------------------------------------------------
	/// \brief	Returns a set of points for the pre-tesselated surface
	/// \return	pointer to the vertex array for this object
	/// 
	XVertList* Points() {
		return &m_CVs;
	}

	//-------------------------------------------------------------------
	/// this function draws the mesh data
	void gl(XMaterialList& mats,
			XVertList* UseVerts,
			XNormList* UseNorms);

protected:

	//-------------------------------------------------------------------
	///	\brief  This function evaluates the normal vectors
	///	
	void GenerateNormals();

	//-------------------------------------------------------------------
	///	\brief  This function evaluates the texture coords
	///	
	void GenerateTexCoords();

	//-------------------------------------------------------------------
	///	\brief  This function evaluates the vertex indices for the triangle
	///			arrays.
	///	
	void GenerateIndices();


	unsigned int m_Material;

	/// wang in a vertex array to make high speed data
	float *m_VertexArray;

	/// wang in a normal array to make high speed data
	float *m_NormalArray;

	/// wang in a uv array to make high speed data. This is
	/// only set when the LOD changes.
	float *m_UvArray;

	/// an array of all the surfaces triangle strip indices.
	/// This is only set when the LOD changes.
	unsigned int *m_IndexArray;

	/// the number of CVs in the U direction
	unsigned short m_NumCvsU;

	/// the number of CVs in the V direction
	unsigned short m_NumCvsV;

	/// the type of the curve in U, 
	/// 0	=	open
	/// 1	=	closed		-	force last set of control points to be where first set are
	/// 2	=	periodic	-	The surface must maintain smoothness across it's surface.
	///							The last degree num of control points must be clamped to the first degree'th.
	///							Err, basically if the u direction is of degree 3, then the last three control
	///							points are clamped to the first three in that direction.
	unsigned char m_TypeU;

	/// the type of the curve in V, 
	/// 0	=	open
	/// 1	=	closed		-	force last set of control points to be where first set are
	/// 2	=	periodic	-	The surface must maintain smoothness across it's surface.
	///							The last degree num of control points must be clamped to the first degree'th.
	///							Err, basically if the u direction is of degree 3, then the last three control
	///							points are clamped to the first three in that direction.
	unsigned char m_TypeV;

	/// the list of CV's in the surface
	XVertList m_CVs;


};

#endif

