
#include <cassert>
#include <stdlib.h>
#include <GL/glut.h>
#include "./SkinCluster.h"
#include "./Mesh.h"
#include "./NurbsSurface.h"



std::istream &operator>>(std::istream& ifs,XSkinCluster& skin)
{
	char buffer[512];
	int temp;

	ifs >> buffer	// name
		>> buffer	// NumInfluences
		>> temp;

	// read the influence ID's
	while (temp--)
	{
		int i;
		ifs >> i;
		skin.m_Influences.push_back(i);
	}

	// read mesh ID
	int id;
	ifs >> buffer
		>> id;

	// sets the mesh or surface to deform
	skin.SetObject(id);

	XBone* BindPosition = skin.GetBone(skin.m_pTheObject->GetParentID());

	// read vertex count
	ifs >> buffer
		>> id
		>> buffer;

	// make sure vertex counts match !!
	if (skin.m_pTheObject->Points()->size() != id) {
		assert(0 && "[Error] Vertex counts on skin cluster don't match those on the surface");
	}

	// one thing we have to do is stop the mesh being drawn directly. To do this simply
	// replace the skin cluster ID with the meshes ID and invalidate the meshes ID.
	skin.m_Model->Parent(skin.m_pTheObject->GetID(),0);
	skin.m_ID = skin.m_pTheObject->GetID();
	skin.m_pTheObject->m_ID = 0;


	// set the number of points in the mesh
	skin.m_Points.resize( id );

	// calculate the world space bind poses.
	skin.CalculateBasePose();

	// get the transform of the meshes bind pose
	const XMatrix& MBind = BindPosition->Base();

	// loop through all vertices on surface
	for (int i=0;i!=id;++i)
	{
		int num;
		ifs >> num;
		
		// store temporary copy of the vertex
		XVector V(	(*(skin.m_pTheObject->Points()))[i].x ,
					(*(skin.m_pTheObject->Points()))[i].y ,
					(*(skin.m_pTheObject->Points()))[i].z  );


		// read in each weight
		while (num--)
		{
			int idx;
			float w;

			// read index & weight
			ifs >> idx >> w;

			XSkinWeight We;
			We.SetWeight(w);
			We.SetBoneID( idx );

			// get the bone from this influence ID
			XBone *B = skin.GetBone( skin.m_Influences[idx] );

			// invert the bones world space matrix
			XMatrix inv;
			if(!skin.m_WorldMatrices[ idx ].Inverse(inv))
			{
				std::cerr << "COULD NOT INVERT MATRIX!!!" << std::endl;
			}

			// calculate the world space position of the vertex when it was actually bound.
			// 
			XVector WSV = MBind*V;

			// we set the local vertex position by multiplying the world space point with
			// the bones inverse transform when bound.
			We.SetVertex( i, (inv*WSV) );

			// store the skin weight.
			skin.m_SkinWeights.push_back(We);
		}
	}

	skin.m_WorldMatrices.resize( skin.m_Influences.size() );


	return ifs;
}


//-------------------------------------------------------------------
///	\brief  Sets the geometry object that this skin cluster will 
///			deform.
///	\param  obj	-	the object ID
///
void XSkinCluster::SetObject(unsigned int obj) {
	m_pTheObject = GetObject(obj);
}


//-------------------------------------------------------------------
///	\brief  Calculates the bind pose of the bone hierarchy. I think 
///			there maya be a slight flaw in that the 'bind pose' doesn't
///			seem to get exported. Instead we have a static animated pose
///			to grab the bind info off instead. Can't be bothered to
///			do anything about it, it doesn't make any difference to the
///			outcome anyway....
///
void XSkinCluster::CalculateBasePose() {

	m_WorldMatrices.resize( m_Influences.size() );

	XBoneList::iterator it = m_Model->m_Bones.begin();

	for( ; it != m_Model->m_Bones.end(); ++it )
	{
		if ( (*it)->GetParentID() == 0)
		{
			glColor3f(0,1,0);
			DoCalc( (*it) );
		}
	}
}

int tbs=0;

//-------------------------------------------------------------------
///	\brief  Calculates the World space 
///	\param   	
///	\return		
///

void XSkinCluster::DoCalc(XBone* B,const XMatrix& WS ) 
{
	assert(B);
	
	for (int i_=0;i_<tbs;++i_)
	{
		std::cout << "  ";
	}

	std::cout << B->name.c_str() << "\n";

	++tbs;


	// get the index of the bone
	int idx = GetBoneIdx(B);

	// calculate the world space, parent transform * local transform
	XMatrix World = WS * B->Base();

	// only store the matrix if it's part of the skin cluster
	if(idx!=-1)
		m_WorldMatrices[idx] =  World;

	// loop through all children
	ChildIt it;
	for( it = B->m_ChildJoints.begin(); it != B->m_ChildJoints.end(); ++it )
	{
		// get the bone
		XBone *Bone = GetBone(*it);
		assert(Bone);

		// calculate it's transforms & stuff
		DoCalc(Bone,World );
	}

	--tbs;
}

int XSkinCluster::GetBoneIdx(XBone* B){
	std::vector<unsigned short>::iterator it = m_Influences.begin();
	for( int i=0 ; it != m_Influences.end(); ++it,++i )
	{
		if( *it == B->GetID() )
		{
			return i;
		}
	}
	return -1;
}



void XSkinCluster::ApplySkinning( XVertList& OutputVertices )
{

	{
		XVertList::iterator it = OutputVertices.begin();
		for( ; it != OutputVertices.end(); ++it )
		{
			it->x = it->y = it->z = 0;
		}
	}

	{
		std::vector< XMatrix >::iterator itw = m_WorldMatrices.begin();
		std::vector< unsigned short >::iterator it =  m_Influences.begin();
		for (; it != m_Influences.end(); ++it,++itw )
		{
			XBone* B = GetBone(*it);
			*itw = B->World();
		}
	}

	// iterate through all of the weights and apply them all to the mesh, 
	// or rather the output verts & norms arrays's.... 
	//
	for(WeightIt_ it = m_SkinWeights.begin(); it != m_SkinWeights.end(); ++it)
	{
		OutputVertices[it->m_VertexID] +=  (m_WorldMatrices[it->m_BoneID] * it->m_LocalSpaceVert) * it->m_fWeight;
	}
}

