///-----------------------------------------------------------------------------
/// 
/// \file	TextureFile.h
/// \author	Rob Bateman
/// \brief	the XTextureFile class holds a list of all loaded textures from the 
///			specified file.
/// \note	
/// 		This sourcefile is provided for free use for non-commercial purposes only. The
/// 		Software is provided 'as is' and no responsibility can be taken for any damages
/// 		that may be caused while using it. If you make use of this software within 
/// 		your code then it would be nice to have a small credit somewhere in your
/// 		finished product for my hard work :(
/// 		An e-mail saying you found it useful and possibly even a link to your product
/// 		would be really nice too ;)
///
///			Slip me a couple of quid and i might be kind enough to offer assistance in
/// 		sorting out your exporters ;) 
///
///				(C) 2004 Rob 'give me a job offer' Bateman 
/// 
/// 
///-----------------------------------------------------------------------------


#ifdef WIN32
#pragma once
#endif

#ifndef __X_TEXTURE_FILE__H__
#define __X_TEXTURE_FILE__H__

#include <vector>
#include "Texture.h"


//------------------------------------------------------------------------------
///	\brief  This class holds a set of textures for use on a model. 
///			The class can load and save a set of textures as a single
///			pack file.	
///
class XTextureFile 
	: public XFn,
	  protected std::vector< XTexture* >
{
public:

	//--------------------------------------------------------------------------
	/// ctor
	XTextureFile();

	//--------------------------------------------------------------------------
	/// dtor
	~XTextureFile();

	//--------------------------------------------------------------------------
	/// \brief	returns the type of this node (XFn::TexturePak)
	/// \return	the type
	XFn::Type GetType() const {
		return XFn::TexturePak;
	}


	//--------------------------------------------------------------------------
	/// \brief	returns a pointer to the texture data
	/// \param	id	-	the id number of the texture to return
	/// 
	XTexture* GetTexture(unsigned int id) const 
	{
		if(id == 0)
			return NULL;

		if (id-1 < size()) {
			return (*this)[id-1];
		}
		return NULL;
	}

	//--------------------------------------------------------------------------
	/// \brief	this function converts all image data into a single openGL
	///			texture object. to use each texture, use tex_obj + n to get the 
	///			n'thy texture object
	/// \param	tex_obj	-	the output texture object
	/// \return	true if all OK
	/// 
	bool TurnIntoTextures(unsigned int& tex_obj);

	//--------------------------------------------------------------------------
	/// \brief	returns the number of textures in the file
	/// \return	the number of textures in the file
	/// 
	unsigned int GetNumTextures() const {
		return static_cast<unsigned int>(size());
	}

	//--------------------------------------------------------------------------
	/// \brief	this function loads the specified file
	/// \param	filename	-	the name of the file to load
	/// \return	true if OK
	///
	bool Load(const char* const filename);

};

#endif


