///-----------------------------------------------------------------------------
/// 
/// \file	main.cpp
/// \author	Rob Bateman
/// \brief	
/// \note	
/// 		This sourcefile is provided for free use for non-commercial purposes only. The
/// 		Software is provided 'as is' and no responsibility can be taken for any damages
/// 		that may be caused while using it. If you make use of this software within 
/// 		your code then it would be nice to have a small credit somewhere in your
/// 		finished product for my hard work :(
/// 		An e-mail saying you found it useful and possibly even a link to your product
/// 		would be really nice too ;)
///
///			Slip me a couple of quid and i might be kind enough to offer assistance in
/// 		sorting out your exporters ;) 
///
///				(C) 2004 Rob 'give me a job offer' Bateman 
///-----------------------------------------------------------------------------

#ifdef WIN32
#include<Windows.h>
#endif

/// this has been added for the frametimer
#include<LibUtil.h>

#include <GL/glut.h>

#include "Model.h"
#include "FileDialog.h"

/// The global model instance
XModel model;

// camera variables
float zoom = 5;
float rotx = 0;
float roty = 0;
float tx = 0;
float ty = 0;
int lastx=0;
int lasty=0;

// mouse button states
unsigned char Buttons[3] = {0};

// mouse handler
void Mouse(int button,int state,int x,int y)
{
	lastx=x;
	lasty=y;
	switch(button)
	{
	case GLUT_LEFT_BUTTON:
			Buttons[0] = ((GLUT_DOWN==state)?1:0);
			break;
	case GLUT_MIDDLE_BUTTON:
			Buttons[1] = ((GLUT_DOWN==state)?1:0);
			break;
	case GLUT_RIGHT_BUTTON:
			Buttons[2] = ((GLUT_DOWN==state)?1:0);
			break;
	default:
		break;		
	}
}

// motion handler
void Motion(int x,int y)
{
	int diffx=x-lastx;
	int diffy=y-lasty;
	lastx=x;
	lasty=y;
	
	if( Buttons[0] && Buttons[1] )
	{
		zoom -= (float) 0.05f * diffx;
	}
	else
	if( Buttons[0] )
	{
		rotx += (float) 0.5f * diffy;
		roty += (float) 0.5f * diffx;		
	}
	else
	if( Buttons[1] )
	{
		tx += (float) 0.05f * diffx;
		ty -= (float) 0.05f * diffy;
	}
}

// initialise
void Init() 
{
	glClearColor(0.65f,0.65f,0.65f,0.0f);
	glEnable(GL_DEPTH_TEST);
	LibUtilInit();
	glEnable(GL_NORMALIZE);
	glEnable(GL_AUTO_NORMAL);
}

// display
void display()
{
	XOptions Opt;

	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	
	glTranslatef(0,0,-zoom);
	glTranslatef(tx,ty,0);
	glRotatef(rotx,1,0,0);
	glRotatef(roty,0,1,0);

	XParametricBase::SetCameraPosition( XVector(-tx,-ty,zoom), XVector() );
		
	glPushMatrix();

		if(Opt.ViewGrid())
		{
			glColor3f(0.45f,0.45f,0.45f);
			glBegin(GL_LINES);
				for (int i=-10;i<=10;++i)
				{
					glVertex3i(i,0,-10);
					glVertex3i(i,0,10);

					glVertex3i(-10,0,i);
					glVertex3i(10,0,i);
				}
			glEnd();
		}

		if(Opt.ViewAxes())
		{
			glBegin(GL_LINES);
				glColor3f(1,0,0);
				glVertex3f(1,0,0);
				glVertex3f(0,0,0);
			
				glColor3f(0,1,0);
				glVertex3f(0,1,0);
				glVertex3f(0,0,0);
			
				glColor3f(0,0,1);
				glVertex3f(0,0,1);
				glVertex3f(0,0,0);
			glEnd();
		}
		glEnable(GL_LIGHTING);

		model.gl();

		glDisable(GL_LIGHTING);
		glDisable(GL_TEXTURE_2D);

	glPopMatrix();

	glutSwapBuffers();
}

// reshape func
void reshape(int x, int y)
{
	if(y==0) y = 1;
	glViewport(0,0,x,y);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45,(float)x/y,0.1,100);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

}

// frame timing
float __t = 0;
int __nf=0;

// anim speed scalar
float scalar=1.0f;

// idle func
void idle()
{
	__t += FrameTime();
	++__nf;
	if(__t > 10) {
		std::cout << (__nf/__t) << std::endl;
		__t  = 0;
		__nf = 0;
	}
	LibUtilIdle();

	// update the models animation
	model.Update(scalar*FrameTime());


	glutPostRedisplay();
}

// key func - just used for changing anim speed
void NormalKey(unsigned char key,int x,int y)
{
	if (key == '-')
	{
		scalar *= 0.5f;
	}
	if (key == '+')
	{
		scalar *= 2.0f;
	}
}

// options 
void SpecialKey(int key,int x,int y)
{
	XOptions Options;
	switch(key) {
	case GLUT_KEY_F1:
		Options.ToggleCurves();
		break;
	case GLUT_KEY_F2:
		Options.ToggleNurbs();
		break;
	case GLUT_KEY_F3:
		Options.ToggleMeshes();
		break;
	case GLUT_KEY_F4:
		Options.ToggleJoints();
		break;
	case GLUT_KEY_F5:
		Options.ToggleLocators();
		break;
	case GLUT_KEY_F6:
		Options.ToggleCollisionObjects();
		break;
	case GLUT_KEY_F7:
		Options.ToggleWireframe();
		break;
	case GLUT_KEY_F8:
		Options.ToggleCulling();
		break;
	case GLUT_KEY_F9:
		Options.ToggleAxes();
		break;
	case GLUT_KEY_F10:
		Options.ToggleGrid();
		break;
	case GLUT_KEY_F11:
		Options.ToggleTextured();
		break;
	case GLUT_KEY_F12:
		Options.ToggleSeperateSpecular();
		break;
	default:
		break;
	}
}


// nasty hack for GTk/win32
#ifdef WIN32
int main(int argc,char** argv)
{

#else
int g_argc=0;
char** g_argv=0;
int main(int argc,char** argv)
{
	g_argc = argc;
	g_argv = argv;

#endif
	std::string infile;
	if( argc == 2 )
		infile = argv[1];
	else
	{
		char buffer[512] = {0};
		if( !OpenFileDialog(buffer) ) {
			return 0;
		}
		infile = buffer;
	}
	glutInit(&argc,argv);
	glutInitDisplayMode(GLUT_DOUBLE|GLUT_RGBA|GLUT_DEPTH);
	glutInitWindowSize(640,640);
	glutInitWindowPosition(100,100);
	glutCreateWindow("hello");


	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutMouseFunc(Mouse);
	glutMotionFunc(Motion);
	glutIdleFunc(idle);
	glutSpecialFunc(SpecialKey);
	glutKeyboardFunc(NormalKey);

	Init();

	// Load the model
	model.Load(infile.c_str());

	glutMainLoop();
}

























