// Colour.h: interface for the XColour class.
//
//////////////////////////////////////////////////////////////////////

#ifndef __COLOUR_H__
#define __COLOUR_H__

#if _MSC_VER > 1000
#pragma once
#endif 

struct XColour  
{
	union {
		struct {
			float r;
			float g;
			float b;
			float a;
		};
		float data[4];
	};
	XColour(float r_,float g_,float b_,float a_=1) 
		: r(r_),g(g_),b(b_),a(a_) {}
	
	XColour() : r(0),g(0),b(0),a(0) {}
	XColour(const XColour& c) 
		: r(c.r),g(c.g),b(c.b),a(c.a) {}

	const XColour& InvertRGB() {
		r = 1.0f-r;
		g = 1.0f-g;
		b = 1.0f-b;
		return *this;
	}
	const XColour& InvertAlpha() {
		a = 1.0f-a;
		return *this;
	}

	const XColour& operator = (const XColour& C) {
		r = C.r;
		g = C.g;
		b = C.b;
		a = C.a;
		return *this;
	}
	const XColour& operator *= (const float& C) {
		r *= C;
		g *= C;
		b *= C;
		a *= C;
		return *this;
	}
	const XColour operator * (const float& C) {
		XColour Col(*this);
		Col.r *= C;
		Col.g *= C;
		Col.b *= C;
		Col.a *= C;
		return Col;
	}
	const XColour operator + (const XColour& C) const {
		XColour Col(*this);
		Col.r = C.r+r;
		Col.g = C.g+g;
		Col.b = C.b+b;
		Col.a = C.a+a;
		return Col;
	}
};

#endif 