//=========================================================================
///
///	\file	Matrix.cpp
/// \author	Rob
///	\brief	Defines the class XMatrix, a class definition for a 4x4 matrix
///
//=========================================================================


//=========================================================================
//	Includes
//=========================================================================
#include "Matrix.h"

//=========================================================================
//	XMatrix implimentation
//=========================================================================

// ctor
XMatrix::XMatrix()
{
	m_01 = m_02 = m_03		=
		m_10 = m_12 = m_13	=
		m_20 = m_21 = m_23	=
		m_30 = m_31 = m_32	= 0.0f;
	m_00 = m_11 = m_22 = m_33 = 1.0f;
}

// dtor
XMatrix::~XMatrix()
{
}



XQuaternion XMatrix::ToQuaternion() const {

	// calculate trace of the matrix
	float T = data[0] + data[5] + data[10] + 1;

	// if trace is greater than 0, calculate an instant calculation
	if( T > 0 ) {
		float S = static_cast<float>( 0.5f / sqrt(T) );
		return XQuaternion( static_cast<float>(( data[6] - data[9] ) * S),
			static_cast<float>( ( data[8] - data[2] ) * S),
			static_cast<float>( ( data[1] - data[4] ) * S),
			static_cast<float>( 0.25f / S) );
	}
	float BigF = data[0];
	unsigned char check=0;
	if( data[5] > BigF ) {
		check = 1;
		BigF = data[5];
	}
	if( data[10] > BigF ) {
		check = 2;
		BigF = data[10];
	}
	switch(check) {
		case 0:
			{
				float S  = static_cast<float>( sqrt( 1.0 + data[0] - data[5] - data[10] ) * 2.0f );

				return XQuaternion( 0.5f / S,
					(data[1] + data[4] ) / S,
					(data[2] + data[8] ) / S,
					(data[6] + data[9] ) / S );
			}
		case 1:
			{
				float S  = static_cast<float>( sqrt( 1.0 + data[5] - data[0] - data[10] ) * 2.0f );

				return XQuaternion((data[1] + data[4] ) / S,
					0.5f / S,
					(data[6] + data[9] ) / S,
					(data[2] + data[8] ) / S );
			}
		case 2:
			{
				float S  = static_cast<float>( sqrt( 1.0 + data[10] - data[0] - data[5] ) * 2.0f );

				return XQuaternion((data[2] + data[8] ) / S,
					(data[6] + data[9] ) / S,
					0.5f / S,
					(data[1] + data[4] ) / S );
			}
		default:
			assert(0 && "SHOULDN'T GET HERE!!!");
			break;
	};
	return XQuaternion();
	//
}

// copy ctor
XMatrix::XMatrix(const XMatrix &m) {
	*this = m;
}


///	\brief	operator *=	XMatrix
///	\param	m	-	the matrix to multiple this by
/// \return	a constant reference to this
///
const XMatrix& XMatrix::operator *= (const XMatrix &m)
{
	XMatrix temp(*this);

	//	row 0
	m_00  =  temp.m_00 * m.m_00;
	m_01  =  temp.m_01 * m.m_00;
	m_02  =  temp.m_02 * m.m_00;
	m_03  =  temp.m_03 * m.m_00;

	m_00 +=  temp.m_10 * m.m_01;
	m_01 +=  temp.m_11 * m.m_01;
	m_02 +=  temp.m_12 * m.m_01;
	m_03 +=  temp.m_13 * m.m_01;

	m_00 +=  temp.m_20 * m.m_02;
	m_01 +=  temp.m_21 * m.m_02;
	m_02 +=  temp.m_22 * m.m_02;
	m_03 +=  temp.m_23 * m.m_02;

	m_00 +=  temp.m_30 * m.m_03;
	m_01 +=  temp.m_31 * m.m_03;
	m_02 +=  temp.m_32 * m.m_03;
	m_03 +=  temp.m_33 * m.m_03;


	//	row 1
	m_10  =  temp.m_00 * m.m_10;
	m_11  =  temp.m_01 * m.m_10;
	m_12  =  temp.m_02 * m.m_10;
	m_13  =  temp.m_03 * m.m_10;

	m_10 +=  temp.m_10 * m.m_11;
	m_11 +=  temp.m_11 * m.m_11;
	m_12 +=  temp.m_12 * m.m_11;
	m_13 +=  temp.m_13 * m.m_11;

	m_10 +=  temp.m_20 * m.m_12;
	m_11 +=  temp.m_21 * m.m_12;
	m_12 +=  temp.m_22 * m.m_12;
	m_13 +=  temp.m_23 * m.m_12;

	m_10 +=  temp.m_30 * m.m_13;
	m_11 +=  temp.m_31 * m.m_13;
	m_12 +=  temp.m_32 * m.m_13;
	m_13 +=  temp.m_33 * m.m_13;


	//	row 2
	m_20  =  temp.m_00 * m.m_20;
	m_21  =  temp.m_01 * m.m_20;
	m_22  =  temp.m_02 * m.m_20;
	m_23  =  temp.m_03 * m.m_20;

	m_20 +=  temp.m_10 * m.m_21;
	m_21 +=  temp.m_11 * m.m_21;
	m_22 +=  temp.m_12 * m.m_21;
	m_23 +=  temp.m_13 * m.m_21;

	m_20 +=  temp.m_20 * m.m_22;
	m_21 +=  temp.m_21 * m.m_22;
	m_22 +=  temp.m_22 * m.m_22;
	m_23 +=  temp.m_23 * m.m_22;

	m_20 +=  temp.m_30 * m.m_23;
	m_21 +=  temp.m_31 * m.m_23;
	m_22 +=  temp.m_32 * m.m_23;
	m_23 +=  temp.m_33 * m.m_23;


	//	row 3
	m_30  =  temp.m_00 * m.m_30;
	m_31  =  temp.m_01 * m.m_30;
	m_32  =  temp.m_02 * m.m_30;
	m_33  =  temp.m_03 * m.m_30;

	m_30 +=  temp.m_10 * m.m_31;
	m_31 +=  temp.m_11 * m.m_31;
	m_32 +=  temp.m_12 * m.m_31;
	m_33 +=  temp.m_13 * m.m_31;

	m_30 +=  temp.m_20 * m.m_32;
	m_31 +=  temp.m_21 * m.m_32;
	m_32 +=  temp.m_22 * m.m_32;
	m_33 +=  temp.m_23 * m.m_32;

	m_30 +=  temp.m_30 * m.m_33;
	m_31 +=  temp.m_31 * m.m_33;
	m_32 +=  temp.m_32 * m.m_33;
	m_33 +=  temp.m_33 * m.m_33;
	return *this;
}

///	\brief	operator	XMatrix  *	XMatrix
///	\param	m	-	the matrix to multiple this by
/// \return	a constant reference to this
///
XMatrix  XMatrix::operator * (const XMatrix &m) const
{
	XMatrix temp = *this;

	// row 0
	temp.m_00  =  m_00 * m.m_00;
	temp.m_01  =  m_01 * m.m_00;
	temp.m_02  =  m_02 * m.m_00;
	temp.m_03  =  m_03 * m.m_00;

	temp.m_00 +=  m_10 * m.m_01;
	temp.m_01 +=  m_11 * m.m_01;
	temp.m_02 +=  m_12 * m.m_01;
	temp.m_03 +=  m_13 * m.m_01;

	temp.m_00 +=  m_20 * m.m_02;
	temp.m_01 +=  m_21 * m.m_02;
	temp.m_02 +=  m_22 * m.m_02;
	temp.m_03 +=  m_23 * m.m_02;

	temp.m_00 +=  m_30 * m.m_03;
	temp.m_01 +=  m_31 * m.m_03;
	temp.m_02 +=  m_32 * m.m_03;
	temp.m_03 +=  m_33 * m.m_03;


	//	row 1
	temp.m_10  =  m_00 * m.m_10;
	temp.m_11  =  m_01 * m.m_10;
	temp.m_12  =  m_02 * m.m_10;
	temp.m_13  =  m_03 * m.m_10;

	temp.m_10 +=  m_10 * m.m_11;
	temp.m_11 +=  m_11 * m.m_11;
	temp.m_12 +=  m_12 * m.m_11;
	temp.m_13 +=  m_13 * m.m_11;

	temp.m_10 +=  m_20 * m.m_12;
	temp.m_11 +=  m_21 * m.m_12;
	temp.m_12 +=  m_22 * m.m_12;
	temp.m_13 +=  m_23 * m.m_12;

	temp.m_10 +=  m_30 * m.m_13;
	temp.m_11 +=  m_31 * m.m_13;
	temp.m_12 +=  m_32 * m.m_13;
	temp.m_13 +=  m_33 * m.m_13;


	//	row 2
	temp.m_20  =  m_00 * m.m_20;
	temp.m_21  =  m_01 * m.m_20;
	temp.m_22  =  m_02 * m.m_20;
	temp.m_23  =  m_03 * m.m_20;

	temp.m_20 +=  m_10 * m.m_21;
	temp.m_21 +=  m_11 * m.m_21;
	temp.m_22 +=  m_12 * m.m_21;
	temp.m_23 +=  m_13 * m.m_21;

	temp.m_20 +=  m_20 * m.m_22;
	temp.m_21 +=  m_21 * m.m_22;
	temp.m_22 +=  m_22 * m.m_22;
	temp.m_23 +=  m_23 * m.m_22;

	temp.m_20 +=  m_30 * m.m_23;
	temp.m_21 +=  m_31 * m.m_23;
	temp.m_22 +=  m_32 * m.m_23;
	temp.m_23 +=  m_33 * m.m_23;


	//	row 3
	temp.m_30  =  m_00 * m.m_30;
	temp.m_31  =  m_01 * m.m_30;
	temp.m_32  =  m_02 * m.m_30;
	temp.m_33  =  m_03 * m.m_30;

	temp.m_30 +=  m_10 * m.m_31;
	temp.m_31 +=  m_11 * m.m_31;
	temp.m_32 +=  m_12 * m.m_31;
	temp.m_33 +=  m_13 * m.m_31;

	temp.m_30 +=  m_20 * m.m_32;
	temp.m_31 +=  m_21 * m.m_32;
	temp.m_32 +=  m_22 * m.m_32;
	temp.m_33 +=  m_23 * m.m_32;

	temp.m_30 +=  m_30 * m.m_33;
	temp.m_31 +=  m_31 * m.m_33;
	temp.m_32 +=  m_32 * m.m_33;
	temp.m_33 +=  m_33 * m.m_33;
	return temp;
}

///	\brief	operator	XMatrix  *	XVector
///	\param	v	-	the vector to transform
/// \return	a constant reference to this
///
XVector XMatrix::operator * (const XVector &v) const
{
	XVector temp;
	#if 0
	temp.x  = m_00 * v.x;
	temp.x += m_01 * v.y;
	temp.x += m_02 * v.z;
	temp.x += m_03 * v.w;

	temp.y  = m_10 * v.x;
	temp.y += m_11 * v.y;
	temp.y += m_12 * v.z;
	temp.y += m_13 * v.w;

	temp.z  = m_20 * v.x;
	temp.z += m_21 * v.y;
	temp.z += m_22 * v.z;
	temp.z += m_23 * v.w;

	temp.w  = m_30 * v.x;
	temp.w += m_31 * v.y;
	temp.w += m_32 * v.z;
	temp.w += m_33 * v.w;
	#else
	temp.x  = m_00 * v.x;
	temp.y  = m_01 * v.x;
	temp.z  = m_02 * v.x;
	temp.w  = m_03 * v.x;
	
	temp.x  += m_10 * v.y;
	temp.y  += m_11 * v.y;
	temp.z  += m_12 * v.y;
	temp.w  += m_13 * v.y;
	
	temp.x  += m_20 * v.z;
	temp.y  += m_21 * v.z;
	temp.z  += m_22 * v.z;
	temp.w  += m_23 * v.z;
	
	temp.x  += m_30 * v.w;
	temp.y  += m_31 * v.w;
	temp.z  += m_32 * v.w;
	temp.w  += m_33 * v.w;
	#endif
	return temp;
}

///	\brief	operator	XVector	*	XMatrix
///	\param	v	-	the vector to transform
///	\param	m	-	the matrix to multiple this by
/// \return	a constant reference to this
///
XVector operator * (const XVector &v,const XMatrix &m)
{
	XVector temp;
	#if 0
	temp.x  = m.m_00 * v.x;
	temp.x += m.m_01 * v.y;
	temp.x += m.m_02 * v.z;
	temp.x += m.m_03 * v.w;

	temp.y  = m.m_10 * v.x;
	temp.y += m.m_11 * v.y;
	temp.y += m.m_12 * v.z;
	temp.y += m.m_13 * v.w;

	temp.z  = m.m_20 * v.x;
	temp.z += m.m_21 * v.y;
	temp.z += m.m_22 * v.z;
	temp.z += m.m_23 * v.w;

	temp.w  = m.m_30 * v.x;
	temp.w += m.m_31 * v.y;
	temp.w += m.m_32 * v.z;
	temp.w += m.m_33 * v.w;
	
	#else
	temp.x  = m.m_00 * v.x;
	temp.y  = m.m_01 * v.x;
	temp.z  = m.m_02 * v.x;
	temp.w  = m.m_03 * v.x;
	
	temp.x  += m.m_10 * v.y;
	temp.y  += m.m_11 * v.y;
	temp.z  += m.m_12 * v.y;
	temp.w  += m.m_13 * v.y;
	
	temp.x  += m.m_20 * v.z;
	temp.y  += m.m_21 * v.z;
	temp.z  += m.m_22 * v.z;
	temp.w  += m.m_23 * v.z;
	
	temp.x  += m.m_30 * v.w;
	temp.y  += m.m_31 * v.w;
	temp.z  += m.m_32 * v.w;
	temp.w  += m.m_33 * v.w;
	
	#endif
	return temp;
}

///	\brief	a function to action a rotation of the specified
///			degrees around the x-axis
/// \param	degrees	-	the amount of the rotation
/// \return	reference to this for concatonation of function calls
/// \note
///
XMatrix& XMatrix::RotateX( const XReal degrees )
{
	XMatrix m;

	XReal  sr = (XReal)sin( (degrees * PI_OVER_180) );
	XReal  cr = (XReal)cos( (degrees * PI_OVER_180) );

	m.m_11 =  cr;
	m.m_21 = -sr;
	m.m_12 =  sr;
	m.m_22 =  cr;

	*this *= m;

	return *this;
}

///	\brief	a function to action a rotation of the specified
///			degrees around the y-axis
/// \param	degrees	-	the amount of the rotation
/// \return	reference to this for concatonation of function calls
/// \note
///
XMatrix& XMatrix::RotateY( const XReal degrees )
{
	XMatrix m;

	XReal  sr = (XReal)sin( (degrees * PI_OVER_180) );
	XReal  cr = (XReal)cos( (degrees * PI_OVER_180) );

	m.m_00 =  cr;

	// i changed this sin to minus and the other one to positive, I may be wrong!
	// it didn't seem to match the matrix & quaternion FAQ so i'll assume that has
	// more intergrity than me....
	m.m_20 =  sr;
	m.m_02 = -sr;
	m.m_22 =  cr;

	*this *= m;

	return *this;
}

///	\brief	a function to action a rotation of the specified
///			degrees around the z-axis
/// \param	degrees	-	the amount of the rotation
/// \return	reference to this for concatonation of function calls
/// \note
///
XMatrix& XMatrix::RotateZ( const XReal degrees )
{
	XMatrix m;

	XReal  sr = (XReal)sin( (degrees * PI_OVER_180) );
	XReal  cr = (XReal)cos( (degrees * PI_OVER_180) );

	m.m_00 =  cr;
	m.m_10 = -sr;
	m.m_01 =  sr;
	m.m_11 =  cr;

	*this *= m;

	return *this;
}

///	\brief	a function to action a scale of (x_,y_,z_)
/// \param	x_	-	the x-component of the scale
/// \param	y_	-	the y-component of the scale
/// \param	z_	-	the z-component of the scale
/// \return	reference to this for concatonation of function calls
/// \note
///
XMatrix& XMatrix::Scale( const XReal x_ ,
						 const XReal y_ ,
						 const XReal z_ )
{
	XMatrix m;

	m.m_00 = static_cast<XReal>(x_);
	m.m_11 = static_cast<XReal>(y_);
	m.m_22 = static_cast<XReal>(z_);

	*this *= m;

	return *this;
}

///	\brief	a function to action a scale of ( sv.x_, sv.y_, sv.z_ )
/// \param	sv	-	the scale as a vector
/// \return	reference to this for concatonation of function calls
/// \note
///
XMatrix& XMatrix::Scale( const XVector &sv)
{
	XMatrix m;

	m.m_00 = sv.x;
	m.m_11 = sv.y;
	m.m_22 = sv.z;

	*this *= m;

	return *this;
}

///	\brief	a function to action a translation of ( x_, y_, z _)
/// \param	x_	-	the x-component of the translation
/// \param	y_	-	the y-component of the translation
/// \param	z_	-	the z-component of the translation
/// \return	reference to this for concatonation of function calls
/// \note
///
XMatrix& XMatrix::Translate( const XReal x_ ,
							 const XReal y_ ,
							 const XReal z_ )
{
	XMatrix m;

	m.m_30 = static_cast<XReal>(x_);
	m.m_31 = static_cast<XReal>(y_);
	m.m_32 = static_cast<XReal>(z_);

	*this *= m;

	return *this;
}

///	\brief	a function to action a translation of ( tv.x_, tv.y_, tv.z_ )
/// \param	tv	-	the translation as a vector
/// \return	reference to this for concatonation of function calls
/// \note
///
XMatrix& XMatrix::Translate( const XVector &tv )
{
	XMatrix m;

	m.m_30 = tv.x;
	m.m_31 = tv.y;
	m.m_32 = tv.z;

	*this *= m;

	return *this;
}


/// \brief	This function returns the inverse of this matrix
///	\return	The inverse of this matrix
/// 
void XMatrix::CheapInverse(XMatrix& result) 
{
	float *ptr = data;

	// The rotational part of the matrix is simply the transpose of the
	// original matrix.
	result.data[0]   = *(  ptr);
	result.data[4]   = *(++ptr);
	result.data[8]   = *(++ptr);

	float Tx		 = *(++ptr);

	result.data[1]   = *(++ptr);
	result.data[5]   = *(++ptr);
	result.data[9]   = *(++ptr);

	float Ty		 = *(++ptr);

	result.data[2]   = *(++ptr);
	result.data[6]   = *(++ptr);
	result.data[10]  = *(++ptr);

	float Tz	     = *(++ptr);

	ptr = data;

	// Result = -(Tm * Rm) to get the translation part of the inverse
	result.m_03  = - *(  ptr) * Tx;
	result.m_13  = - *(++ptr) * Tx; 
	result.m_23  = - *(++ptr) * Tx;
	++ptr;
	result.m_03  -=  *(++ptr) * Ty;
	result.m_13  -=  *(++ptr) * Ty;
	result.m_23  -=  *(++ptr) * Ty;
	++ptr;
	result.m_03  -=  *(++ptr) * Tz;
	result.m_13  -=  *(++ptr) * Tz;
	result.m_23  -=  *(++ptr) * Tz;
}


void XMatrix::SubMatrix3x3(int i,int j,XReal mat[]) const
{
	int ti, tj, idst, jdst;

	for ( ti = 0; ti < 4; ti++ )
	{
		if ( ti < i )
		{
			idst = ti;
		}
		else
		if ( ti > i )
		{
			idst = ti-1;
		}

		for ( tj = 0; tj < 4; tj++ )
		{
			if ( tj < j )
			{
				jdst = tj;
			}
			else
			if ( tj > j )
			{
				jdst = tj-1;
			}

			if ( ti != i && tj != j )
			{
				mat[idst*3 + jdst] = data[ti*4 + tj];
			}
		}
	}
}

XReal XMatrix::Determinant() const
{
	XReal det, result = 0, i = 1;
	XReal matrixSub3x3[9];

	for( int n = 0; n < 4; n++, i *= -1 )
	{
		SubMatrix3x3( 0, n, matrixSub3x3 );

		det =   matrixSub3x3[0] * ( matrixSub3x3[4]*matrixSub3x3[8] - matrixSub3x3[7]*matrixSub3x3[5] )
			  - matrixSub3x3[1] * ( matrixSub3x3[3]*matrixSub3x3[8] - matrixSub3x3[6]*matrixSub3x3[5] )
              + matrixSub3x3[2] * ( matrixSub3x3[3]*matrixSub3x3[7] - matrixSub3x3[6]*matrixSub3x3[4] );

		result += data[n] * det * i;
	}

	return result;
}

bool XMatrix::Inverse(XMatrix& result) const
{
	XReal determinant = Determinant();
	XReal matrixSub3x3[9];

	int i, j, sign;

	// if the determinant is 0 (or as near as makes no difference), we cannot 
	// calculate the inverse
	if ( fabs(determinant) < 0.0005 )
		return false;

	for ( i = 0; i < 4; i++ )
	{
		for ( j = 0; j < 4; j++ )
		{
			sign = 1 - ( (i +j) % 2 ) * 2;

			SubMatrix3x3( i, j, matrixSub3x3 );


			XReal det =   matrixSub3x3[0] * ( matrixSub3x3[4]*matrixSub3x3[8] - matrixSub3x3[7]*matrixSub3x3[5] )
						- matrixSub3x3[1] * ( matrixSub3x3[3]*matrixSub3x3[8] - matrixSub3x3[6]*matrixSub3x3[5] )
						+ matrixSub3x3[2] * ( matrixSub3x3[3]*matrixSub3x3[7] - matrixSub3x3[6]*matrixSub3x3[4] );

			result.data[i+j*4] = ( det * sign ) / determinant;
		}
	}
	return true;
}



///	\brief	operator  std::istream >> XMatrix
///	\param	ifs	-	the input file stream, ie std::cin
///	\param	m	-	the matrix instance into which the data is being placed.
/// \return	the stream reference
/// \note	The stream extraction operator has to be declared as a friend of the
///			class. This is because the operator itself cannot be a member function
///			of XMatrix because the type, std::istream, appears on the left of the
///			operator. For it to be a member function, the XMatrix would have to be
///			on the left of the >> operator. This forces us to make the operator act
///			globally and should therefore always be declared as a friend.
///
std::istream &operator >> (std::istream &ifs,XMatrix &m)
{
	ifs >> m.m_00 >> m.m_01 >> m.m_02 >> m.m_03;
	ifs	>> m.m_10 >> m.m_11 >> m.m_12 >> m.m_13;
	ifs	>> m.m_20 >> m.m_21 >> m.m_22 >> m.m_23;
	ifs	>> m.m_30 >> m.m_31 >> m.m_32 >> m.m_33;
	return ifs;
}

///	\brief	operator  std::ostream << XMatrix
///	\param	ofs	-	the output file stream, ie std::cout, std::fstream etc
///	\param	m	-	the data instance to output
/// \return	the stream reference
/// \note	The stream insertion operator has to be declared as a friend of the
///			class. This is because the operator itself cannot be a member function
///			of XMatrix because the type, std::ostream, appears on the left of the
///			operator. For it to be a member function, the XMatrix would have to be
///			on the left of the << operator. This forces us to make the operator act
///			globally and should therefore always be declared as a friend.
///
/*
std::ostream &operator << (std::ostream &ofs,XMatrix &m)
{
	ofs << m.m_00 << " ";
	ofs << m.m_01 << " ";
	ofs << m.m_02 << " ";
	ofs << m.m_03 << "\n";

	ofs << m.m_10 << " ";
	ofs << m.m_11 << " ";
	ofs << m.m_12 << " ";
	ofs << m.m_13 << "\n";

	ofs << m.m_20 << " ";
	ofs << m.m_21 << " ";
	ofs << m.m_22 << " ";
	ofs << m.m_23 << "\n";

	ofs << m.m_30 << " ";
	ofs << m.m_31 << " ";
	ofs << m.m_32 << " ";
	ofs << m.m_33 << "\n";
	return ofs;
}*/





