//=========================================================================
///
///	\file	Plane.cpp
/// \author	Rob
///	\brief	Defines the class XPlane, a class definition for a 3D plane of
///			the form
///
///			ax + by + cz + d = 0
///
//=========================================================================

#include "Plane.h"

/// This function intersects the polygon defined by inpoints by this plane. The 
/// resulting polygon is stored within outpoints. 
/// 
void XPlane::CutPoly(XVecList &outpoints,const XVecList& inpoints) const {
	
	// get iterator to the points
	XCItVec it = inpoints.begin();
	
	// check  the first point and store the result for later
	bool FirstPoint = Distance(*it) > 0,LastPoint;
	
	// using the last point variable to see if a line has crossed the plane boundary
	LastPoint = FirstPoint;
	
	// if first point infront of poly, add it to the list
	if( FirstPoint )
		outpoints.push_back(*it);
	++it;
	
	// loop through rest of points
	for( ; it != inpoints.end(); ++it )
	{
		// check this point against the plane. Is it infront?
		bool LP = Distance(*it) > 0;
		if( LP )
		{
			// if the last point was on the other side
			if(!LastPoint)
			{
				// add the intersect point to the list
				outpoints.push_back( GetLineIntersect( *it, *(it-1) ) );
			}
			// add the current point also
			outpoints.push_back(*it);
		}
		else {
			
			// if we have another intersect, add the intersection
			if(LastPoint)
			{
				outpoints.push_back( GetLineIntersect( *(it-1), *it ) );
			}
		}
		
		// store result of this point's test for next loop
		LastPoint = LP;
	}
	
	// if an intersect exists between the first and last point
	if( (!LastPoint && FirstPoint) )
	{
		// add final intersect
		outpoints.push_back( GetLineIntersect( inpoints[0], inpoints[inpoints.size()-1] ) );
	}
	// if an intersect exists between the first and last point
	if( (!FirstPoint && LastPoint) )
	{
		// add final intersect
		outpoints.push_back( GetLineIntersect( inpoints[inpoints.size()-1],inpoints[0] ) );
	}
}

